@@ -326,7 +326,6 @@ static BOOL nulldrv_GetCharWidth( PHYSDEV dev, UINT first, UINT last, INT *buffe
 
 static INT nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
 {
-    static int screen_dpi;
     int bpp;
 
     switch (cap)
@@ -372,16 +371,14 @@ static INT nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
     case PHYSICALOFFSETY: return 0;
     case SCALINGFACTORX:  return 0;
     case SCALINGFACTORY:  return 0;
-    case VREFRESH:        return 0;
+    case VREFRESH:        return GetDeviceCaps( dev->hdc, TECHNOLOGY ) == DT_RASDISPLAY ? 1 : 0;
     case DESKTOPVERTRES:  return GetDeviceCaps( dev->hdc, VERTRES );
     case DESKTOPHORZRES:  return GetDeviceCaps( dev->hdc, HORZRES );
     case BLTALIGNMENT:    return 0;
     case SHADEBLENDCAPS:  return 0;
     case COLORMGMTCAPS:   return 0;
     case LOGPIXELSX:
-    case LOGPIXELSY:
-        if (!screen_dpi && !(screen_dpi = get_dpi())) screen_dpi = 96;
-        return screen_dpi;
+    case LOGPIXELSY:      return get_system_dpi();
     case NUMCOLORS:
         bpp = GetDeviceCaps( dev->hdc, BITSPIXEL );
         return (bpp > 8) ? -1 : (1 << bpp);
@@ -709,6 +706,11 @@ static struct opengl_funcs *nulldrv_wine_get_wgl_driver( PHYSDEV dev, UINT versi
     return (void *)-1;
 }
 
+static const struct vulkan_funcs *nulldrv_wine_get_vulkan_driver( PHYSDEV dev, UINT version )
+{
+    return NULL;
+}
+
 const struct gdi_dc_funcs null_driver =
 {
     nulldrv_AbortDoc,                   /* pAbortDoc */
@@ -838,6 +840,7 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_UnrealizePalette,           /* pUnrealizePalette */
     nulldrv_WidenPath,                  /* pWidenPath */
     nulldrv_wine_get_wgl_driver,        /* wine_get_wgl_driver */
+    nulldrv_wine_get_vulkan_driver,     /* wine_get_vulkan_driver */
 
     GDI_PRIORITY_NULL_DRV               /* priority */
 };
