@@ -420,7 +420,7 @@ static BOOL UDF_Find_PVD( HANDLE handle, BYTE pvd[] )
     DWORD offset;
     INT locations[] = { 256, -1, -257, 512 };
 
-    for(i=0; i<sizeof(locations)/sizeof(locations[0]); i++)
+    for(i=0; i<ARRAY_SIZE(locations); i++)
     {
         if (!VOLUME_ReadCDBlock(handle, pvd, locations[i]*BLOCK_SIZE))
             return FALSE;
@@ -994,7 +994,7 @@ BOOL WINAPI GetVolumeNameForVolumeMountPointA( LPCSTR path, LPSTR volume, DWORD
 {
     BOOL ret;
     WCHAR volumeW[50], *pathW = NULL;
-    DWORD len = min( sizeof(volumeW) / sizeof(WCHAR), size );
+    DWORD len = min(ARRAY_SIZE(volumeW), size );
 
     TRACE("(%s, %p, %x)\n", debugstr_a(path), volume, size);
 
@@ -1117,7 +1117,7 @@ BOOL WINAPI GetVolumeNameForVolumeMountPointW( LPCWSTR path, LPWSTR volume, DWOR
             debugstr_wn((WCHAR*)((char *)output + o1->DeviceNameOffset),
                             o1->DeviceNameLength/sizeof(WCHAR)));
 
-        if (!strncmpW( p, volumeW, sizeof(volumeW)/sizeof(WCHAR) ))
+        if (!strncmpW( p, volumeW, ARRAY_SIZE( volumeW )))
         {
             /* is there space in the return variable ?? */
             if ((o1->SymbolicLinkNameLength/sizeof(WCHAR))+2 > size)
@@ -1720,6 +1720,10 @@ BOOL WINAPI GetVolumePathNameW(LPCWSTR filename, LPWSTR volumepathname, DWORD bu
         return FALSE;
     }
     strcpyW( volumenameW, filename );
+
+    /* Normalize path */
+    for (c = volumenameW; *c; c++) if (*c == '/') *c = '\\';
+
     stop_pos = 0;
     /* stop searching slashes early for NT-type and nearly NT-type paths */
     if (strncmpW(ntprefixW, filename, strlenW(ntprefixW)) == 0)
@@ -1786,7 +1790,7 @@ BOOL WINAPI GetVolumePathNameW(LPCWSTR filename, LPWSTR volumepathname, DWORD bu
                 goto cleanup;
             }
         }
-        else if (GetCurrentDirectoryW( sizeof(cwdW)/sizeof(cwdW[0]), cwdW ))
+        else if (GetCurrentDirectoryW(ARRAY_SIZE(cwdW), cwdW ))
         {
             /* if the path is completely bogus then revert to the drive of the working directory */
             fallbackpathW[0] = cwdW[0];
@@ -1803,14 +1807,10 @@ BOOL WINAPI GetVolumePathNameW(LPCWSTR filename, LPWSTR volumepathname, DWORD bu
 
     if (last_pos + 1 <= buflen)
     {
-        WCHAR *p;
         memcpy(volumepathname, filename, last_pos * sizeof(WCHAR));
         if (last_pos + 2 <= buflen) volumepathname[last_pos++] = '\\';
         volumepathname[last_pos] = '\0';
 
-        /* Normalize path */
-        for (p = volumepathname; *p; p++) if (*p == '/') *p = '\\';
-
         /* DOS-style paths always return upper-case drive letters */
         if (volumepathname[1] == ':')
             volumepathname[0] = toupperW(volumepathname[0]);
@@ -1969,12 +1969,12 @@ BOOL WINAPI GetVolumePathNamesForVolumeNameW(LPCWSTR volumename, LPWSTR volumepa
             linkname = (const WCHAR *)((const char *)link + link->MountPoints[j].SymbolicLinkNameOffset);
 
             if (link->MountPoints[j].SymbolicLinkNameLength == sizeof(dosdevicesW) + 2 * sizeof(WCHAR) &&
-                !memicmpW( linkname, dosdevicesW, sizeof(dosdevicesW) / sizeof(WCHAR) ))
+                !memicmpW( linkname, dosdevicesW, ARRAY_SIZE( dosdevicesW )))
             {
                 len += 4;
                 if (volumepathname && len < buflen)
                 {
-                    path[0] = linkname[sizeof(dosdevicesW) / sizeof(WCHAR)];
+                    path[0] = linkname[ARRAY_SIZE( dosdevicesW )];
                     path[1] = ':';
                     path[2] = '\\';
                     path[3] = 0;
