@@ -49,6 +49,7 @@
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL( mscoree );
+WINE_DECLARE_DEBUG_CHANNEL(winediag);
 
 static HINSTANCE MSCOREE_hInstance;
 
@@ -309,6 +310,16 @@ HRESULT WINAPI GetCORVersion(LPWSTR pbuffer, DWORD cchBuffer, DWORD *dwLength)
     return ret;
 }
 
+HRESULT WINAPI CorIsLatestSvc(int *unk1, int *unk2)
+{
+    ERR_(winediag)("If this function is called, it is likely the result of a broken .NET installation\n");
+
+    if (!unk1 || !unk2)
+        return E_POINTER;
+
+    return S_OK;
+}
+
 HRESULT WINAPI GetRequestedRuntimeInfo(LPCWSTR pExe, LPCWSTR pwszVersion, LPCWSTR pConfigurationFile,
     DWORD startupFlags, DWORD runtimeInfoFlags, LPWSTR pDirectory, DWORD dwDirectory, DWORD *dwDirectoryLength,
     LPWSTR pVersion, DWORD cchBuffer, DWORD *dwlength)
@@ -551,12 +562,6 @@ BOOL WINAPI StrongNameSignatureVerificationEx(LPCWSTR filename, BOOL forceVerifi
     return FALSE;
 }
 
-HRESULT WINAPI CreateConfigStream(LPCWSTR filename, IStream **stream)
-{
-    FIXME("(%s, %p): stub\n", debugstr_w(filename), stream);
-    return E_NOTIMPL;
-}
-
 HRESULT WINAPI CreateDebuggingInterfaceFromVersion(int nDebugVersion, LPCWSTR version, IUnknown **ppv)
 {
     const WCHAR v2_0[] = {'v','2','.','0','.','5','0','7','2','7',0};
@@ -664,8 +669,10 @@ static BOOL install_wine_mono(void)
 {
     BOOL is_wow64 = FALSE;
     HMODULE hmsi;
+    UINT (WINAPI *pMsiEnumRelatedProductsA)(LPCSTR,DWORD,DWORD,LPSTR);
     UINT (WINAPI *pMsiGetProductInfoA)(LPCSTR,LPCSTR,LPSTR,DWORD*);
     char versionstringbuf[15];
+    char productcodebuf[39];
     UINT res;
     DWORD buffer_size;
     PROCESS_INFORMATION pi;
@@ -675,8 +682,8 @@ static BOOL install_wine_mono(void)
     LONG len;
     BOOL ret;
 
-    static const char* mono_version = "4.7.0";
-    static const char* mono_product_code = "{E45D8920-A758-4088-B6C6-31DBB276992E}";
+    static const char* mono_version = "4.7.3";
+    static const char* mono_upgrade_code = "{DE624609-C6B5-486A-9274-EF0B854F6BC5}";
 
     static const WCHAR controlW[] = {'\\','c','o','n','t','r','o','l','.','e','x','e',0};
     static const WCHAR argsW[] =
@@ -698,11 +705,22 @@ static BOOL install_wine_mono(void)
         return FALSE;
     }
 
-    pMsiGetProductInfoA = (void*)GetProcAddress(hmsi, "MsiGetProductInfoA");
+    pMsiEnumRelatedProductsA = (void*)GetProcAddress(hmsi, "MsiEnumRelatedProductsA");
 
-    buffer_size = sizeof(versionstringbuf);
+    res = pMsiEnumRelatedProductsA(mono_upgrade_code, 0, 0, productcodebuf);
 
-    res = pMsiGetProductInfoA(mono_product_code, "VersionString", versionstringbuf, &buffer_size);
+    if (res == ERROR_SUCCESS)
+    {
+        pMsiGetProductInfoA = (void*)GetProcAddress(hmsi, "MsiGetProductInfoA");
+
+        buffer_size = sizeof(versionstringbuf);
+
+        res = pMsiGetProductInfoA(productcodebuf, "VersionString", versionstringbuf, &buffer_size);
+    }
+    else if (res != ERROR_NO_MORE_ITEMS)
+    {
+        ERR("MsiEnumRelatedProducts failed, err=%u\n", res);
+    }
 
     FreeLibrary(hmsi);
 
@@ -733,7 +751,7 @@ static BOOL install_wine_mono(void)
         }
     }
 
-    len = GetSystemDirectoryW(app, MAX_PATH-sizeof(controlW)/sizeof(WCHAR));
+    len = GetSystemDirectoryW(app, MAX_PATH - ARRAY_SIZE(controlW));
     memcpy(app+len, controlW, sizeof(controlW));
 
     args = HeapAlloc(GetProcessHeap(), 0, (len*sizeof(WCHAR) + sizeof(controlW) + sizeof(argsW)));
@@ -741,7 +759,7 @@ static BOOL install_wine_mono(void)
         return FALSE;
 
     memcpy(args, app, len*sizeof(WCHAR) + sizeof(controlW));
-    memcpy(args + len + sizeof(controlW)/sizeof(WCHAR)-1, argsW, sizeof(argsW));
+    memcpy(args + len + ARRAY_SIZE(controlW) - 1, argsW, sizeof(argsW));
 
     TRACE("starting %s\n", debugstr_w(args));
 
