@@ -373,8 +373,9 @@ MSVCRT_size_t CDECL _msize(void* mem)
   return size;
 }
 
+#if _MSVCR_VER>=80
 /*********************************************************************
- * _aligned_msize (MSVCR100.@)
+ * _aligned_msize (MSVCR80.@)
  */
 size_t CDECL _aligned_msize(void *p, MSVCRT_size_t alignment, MSVCRT_size_t offset)
 {
@@ -388,6 +389,7 @@ size_t CDECL _aligned_msize(void *p, MSVCRT_size_t alignment, MSVCRT_size_t offs
     alloc_ptr = SAVED_PTR(p);
     return _msize(*alloc_ptr)-alignment-sizeof(void*);
 }
+#endif
 
 /*********************************************************************
  *		calloc (MSVCRT.@)
@@ -405,6 +407,7 @@ void* CDECL MSVCRT_calloc(MSVCRT_size_t count, MSVCRT_size_t size)
   return msvcrt_heap_alloc(HEAP_ZERO_MEMORY, bytes);
 }
 
+#if _MSVCR_VER>=140
 /*********************************************************************
  *		_calloc_base (UCRTBASE.@)
  */
@@ -412,6 +415,7 @@ void* CDECL _calloc_base(MSVCRT_size_t count, MSVCRT_size_t size)
 {
   return MSVCRT_calloc(count, size);
 }
+#endif
 
 /*********************************************************************
  *		free (MSVCRT.@)
@@ -421,6 +425,7 @@ void CDECL MSVCRT_free(void* ptr)
   msvcrt_heap_free(ptr);
 }
 
+#if _MSVCR_VER>=140
 /*********************************************************************
  *		_free_base (UCRTBASE.@)
  */
@@ -428,6 +433,7 @@ void CDECL _free_base(void* ptr)
 {
   msvcrt_heap_free(ptr);
 }
+#endif
 
 /*********************************************************************
  *                  malloc (MSVCRT.@)
@@ -459,6 +465,7 @@ void* CDECL MSVCRT_malloc(MSVCRT_size_t size)
   return ret;
 }
 
+#if _MSVCR_VER>=140
 /*********************************************************************
  *                  _malloc_base (UCRTBASE.@)
  */
@@ -466,6 +473,7 @@ void* CDECL _malloc_base(MSVCRT_size_t size)
 {
   return MSVCRT_malloc(size);
 }
+#endif
 
 /*********************************************************************
  *		realloc (MSVCRT.@)
@@ -478,6 +486,7 @@ void* CDECL MSVCRT_realloc(void* ptr, MSVCRT_size_t size)
   return NULL;
 }
 
+#if _MSVCR_VER>=140
 /*********************************************************************
  *		_realloc_base (UCRTBASE.@)
  */
@@ -485,9 +494,11 @@ void* CDECL _realloc_base(void* ptr, MSVCRT_size_t size)
 {
   return MSVCRT_realloc(ptr, size);
 }
+#endif
 
+#if _MSVCR_VER>=80
 /*********************************************************************
- * _recalloc (MSVCR100.@)
+ * _recalloc (MSVCR80.@)
  */
 void* CDECL _recalloc(void *mem, MSVCRT_size_t num, MSVCRT_size_t size)
 {
@@ -510,6 +521,7 @@ void* CDECL _recalloc(void *mem, MSVCRT_size_t num, MSVCRT_size_t size)
         memset((BYTE*)ret+old_size, 0, size-old_size);
     return ret;
 }
+#endif
 
 /*********************************************************************
  *		__p__amblksiz (MSVCRT.@)
@@ -758,6 +770,7 @@ int CDECL MSVCRT_memmove_s(void *dest, MSVCRT_size_t numberOfElements, const voi
     return 0;
 }
 
+#if _MSVCR_VER>=100
 /*********************************************************************
  *              wmemmove_s (MSVCR100.@)
  */
@@ -780,6 +793,7 @@ int CDECL wmemmove_s(MSVCRT_wchar_t *dest, MSVCRT_size_t numberOfElements,
     memmove(dest, src, sizeof(MSVCRT_wchar_t)*count);
     return 0;
 }
+#endif
 
 /*********************************************************************
  *		memcpy_s (MSVCRT.@)
@@ -803,10 +817,11 @@ int CDECL MSVCRT_memcpy_s(void *dest, MSVCRT_size_t numberOfElements, const void
         return MSVCRT_ERANGE;
     }
 
-    memcpy(dest, src, count);
+    memmove(dest, src, count);
     return 0;
 }
 
+#if _MSVCR_VER>=100
 /*********************************************************************
  *              wmemcpy_s (MSVCR100.@)
  */
@@ -829,9 +844,10 @@ int CDECL wmemcpy_s(MSVCRT_wchar_t *dest, MSVCRT_size_t numberOfElements,
         return MSVCRT_ERANGE;
     }
 
-    memcpy(dest, src, sizeof(MSVCRT_wchar_t)*count);
+    memmove(dest, src, sizeof(MSVCRT_wchar_t)*count);
     return 0;
 }
+#endif
 
 /*********************************************************************
  *		strncpy_s (MSVCRT.@)
