@@ -277,6 +277,19 @@ static inline unsigned int wait_reply( struct __server_request_info *req )
 }
 
 
+/***********************************************************************
+ *           server_call_unlocked
+ */
+unsigned int server_call_unlocked( void *req_ptr )
+{
+    struct __server_request_info * const req = req_ptr;
+    unsigned int ret;
+
+    if ((ret = send_request( req ))) return ret;
+    return wait_reply( req );
+}
+
+
 /***********************************************************************
  *           wine_server_call (NTDLL.@)
  *
@@ -301,13 +314,11 @@ static inline unsigned int wait_reply( struct __server_request_info *req )
  */
 unsigned int wine_server_call( void *req_ptr )
 {
-    struct __server_request_info * const req = req_ptr;
     sigset_t old_set;
     unsigned int ret;
 
     pthread_sigmask( SIG_BLOCK, &server_block_set, &old_set );
-    ret = send_request( req );
-    if (!ret) ret = wait_reply( req );
+    ret = server_call_unlocked( req_ptr );
     pthread_sigmask( SIG_SETMASK, &old_set, NULL );
     return ret;
 }
@@ -797,7 +808,6 @@ static int receive_fd( obj_handle_t *handle )
 /***********************************************************************/
 /* fd cache support */
 
-#include "pshpack1.h"
 union fd_cache_entry
 {
     LONG64 data;
@@ -809,7 +819,6 @@ union fd_cache_entry
         unsigned int        options : 24;
     } s;
 };
-#include "poppack.h"
 
 C_ASSERT( sizeof(union fd_cache_entry) == sizeof(LONG64) );
 
@@ -1115,9 +1124,10 @@ static void start_server(void)
  *
  * Setup the wine configuration dir.
  */
-static void setup_config_dir(void)
+static int setup_config_dir(void)
 {
     const char *p, *config_dir = wine_get_config_dir();
+    int fd_cwd = open( ".", O_RDONLY );
 
     if (chdir( config_dir ) == -1)
     {
@@ -1145,7 +1155,7 @@ static void setup_config_dir(void)
 
     if (mkdir( "dosdevices", 0777 ) == -1)
     {
-        if (errno == EEXIST) return;
+        if (errno == EEXIST) goto done;
         fatal_perror( "cannot create %s/dosdevices\n", config_dir );
     }
 
@@ -1156,11 +1166,15 @@ static void setup_config_dir(void)
 #ifndef __ANDROID__
     symlink( "/", "dosdevices/z:" );
 #else
-    mkdir( "drive_c/windows", 0777 );
-    mkdir( "drive_c/windows/Fonts", 0777 );
-    symlink( "/system/fonts", "drive_c/windows/Fonts/systemfonts" );
-    symlink( "../../../../../share/wine/fonts", "drive_c/windows/Fonts/sharefonts" );
+    mkdir( "drive_c/fonts", 0777 );
+    symlink( "/system/fonts", "drive_c/fonts/system" );
+    symlink( "../../../../share/wine/fonts", "drive_c/fonts/wine" );
 #endif
+
+done:
+    if (fd_cwd == -1) fd_cwd = open( "dosdevices/c:", O_RDONLY );
+    fcntl( fd_cwd, F_SETFD, FD_CLOEXEC );
+    return fd_cwd;
 }
 
 
@@ -1210,11 +1224,7 @@ static int server_connect(void)
     struct stat st;
     int s, slen, retry, fd_cwd;
 
-    /* retrieve the current directory */
-    fd_cwd = open( ".", O_RDONLY );
-    if (fd_cwd != -1) fcntl( fd_cwd, F_SETFD, FD_CLOEXEC );
-
-    setup_config_dir();
+    fd_cwd = setup_config_dir();
     serverdir = wine_get_server_dir();
 
     /* chdir to the server directory */
@@ -1428,11 +1438,13 @@ void server_init_process(void)
 /***********************************************************************
  *           server_init_process_done
  */
-NTSTATUS server_init_process_done(void)
+void server_init_process_done(void)
 {
     PEB *peb = NtCurrentTeb()->Peb;
     IMAGE_NT_HEADERS *nt = RtlImageNtHeader( peb->ImageBaseAddress );
+    void *entry = (char *)peb->ImageBaseAddress + nt->OptionalHeader.AddressOfEntryPoint;
     NTSTATUS status;
+    int suspend;
 
     /* Install signal handlers; this cannot be done earlier, since we cannot
      * send exceptions to the debugger before the create process event that
@@ -1449,13 +1461,15 @@ NTSTATUS server_init_process_done(void)
 #ifdef __i386__
         req->ldt_copy = wine_server_client_ptr( &wine_ldt_copy );
 #endif
-        req->entry    = wine_server_client_ptr( (char *)peb->ImageBaseAddress + nt->OptionalHeader.AddressOfEntryPoint );
+        req->entry    = wine_server_client_ptr( entry );
         req->gui      = (nt->OptionalHeader.Subsystem != IMAGE_SUBSYSTEM_WINDOWS_CUI);
         status = wine_server_call( req );
+        suspend = reply->suspend;
     }
     SERVER_END_REQ;
 
-    return status;
+    assert( !status );
+    signal_start_process( entry, suspend );
 }
 
 
@@ -1464,7 +1478,7 @@ NTSTATUS server_init_process_done(void)
  *
  * Send an init thread request. Return 0 if OK.
  */
-size_t server_init_thread( void *entry_point )
+size_t server_init_thread( void *entry_point, BOOL *suspend )
 {
     static const char *cpu_names[] = { "x86", "x86_64", "PowerPC", "ARM", "ARM64" };
     static const BOOL is_win64 = (sizeof(void *) > sizeof(int));
@@ -1505,6 +1519,7 @@ size_t server_init_thread( void *entry_point )
         info_size         = reply->info_size;
         server_start_time = reply->server_start;
         server_cpus       = reply->all_cpus;
+        *suspend          = reply->suspend;
     }
     SERVER_END_REQ;
 
