@@ -96,8 +96,7 @@ static char *ntlm_GetUsernameArg(LPCWSTR userW, INT userW_length)
 
     unixcp_size =  WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS,
         userW, userW_length, NULL, 0, NULL, NULL) + sizeof(username_arg);
-    user = HeapAlloc(GetProcessHeap(), 0, unixcp_size);
-    if (!user) return NULL;
+    if (!(user = heap_alloc(unixcp_size))) return NULL;
     memcpy(user, username_arg, sizeof(username_arg) - 1);
     WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, userW, userW_length,
         user + sizeof(username_arg) - 1,
@@ -114,8 +113,7 @@ static char *ntlm_GetDomainArg(LPCWSTR domainW, INT domainW_length)
 
     unixcp_size = WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS,
         domainW, domainW_length, NULL, 0,  NULL, NULL) + sizeof(domain_arg);
-    domain = HeapAlloc(GetProcessHeap(), 0, unixcp_size);
-    if (!domain) return NULL;
+    if (!(domain = heap_alloc(unixcp_size))) return NULL;
     memcpy(domain, domain_arg, sizeof(domain_arg) - 1);
     WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, domainW,
         domainW_length, domain + sizeof(domain_arg) - 1,
@@ -127,13 +125,13 @@ static char *ntlm_GetDomainArg(LPCWSTR domainW, INT domainW_length)
 /***********************************************************************
  *              AcquireCredentialsHandleW
  */
-SECURITY_STATUS SEC_ENTRY ntlm_AcquireCredentialsHandleW(
+static SECURITY_STATUS SEC_ENTRY ntlm_AcquireCredentialsHandleW(
  SEC_WCHAR *pszPrincipal, SEC_WCHAR *pszPackage, ULONG fCredentialUse,
  PLUID pLogonID, PVOID pAuthData, SEC_GET_KEY_FN pGetKeyFn,
  PVOID pGetKeyArgument, PCredHandle phCredential, PTimeStamp ptsExpiry)
 {
-    SECURITY_STATUS ret;
-    PNtlmCredentials ntlm_cred;
+    SECURITY_STATUS ret = SEC_E_INSUFFICIENT_MEMORY;
+    PNtlmCredentials ntlm_cred = NULL;
     LPWSTR domain = NULL, user = NULL, password = NULL;
     PSEC_WINNT_AUTH_IDENTITY_W auth_data = NULL;
 
@@ -141,149 +139,117 @@ SECURITY_STATUS SEC_ENTRY ntlm_AcquireCredentialsHandleW(
      debugstr_w(pszPrincipal), debugstr_w(pszPackage), fCredentialUse,
      pLogonID, pAuthData, pGetKeyFn, pGetKeyArgument, phCredential, ptsExpiry);
 
-    switch(fCredentialUse)
+    switch (fCredentialUse)
     {
-        case SECPKG_CRED_INBOUND:
-            ntlm_cred = HeapAlloc(GetProcessHeap(), 0, sizeof(*ntlm_cred));
-            if (!ntlm_cred)
-                ret = SEC_E_INSUFFICIENT_MEMORY;
+    case SECPKG_CRED_INBOUND:
+        if (!(ntlm_cred = heap_alloc(sizeof(*ntlm_cred)))) return SEC_E_INSUFFICIENT_MEMORY;
+        ntlm_cred->mode = NTLM_SERVER;
+        ntlm_cred->username_arg = NULL;
+        ntlm_cred->domain_arg = NULL;
+        ntlm_cred->password = NULL;
+        ntlm_cred->pwlen = 0;
+        ntlm_cred->no_cached_credentials = 0;
+
+        phCredential->dwUpper = fCredentialUse;
+        phCredential->dwLower = (ULONG_PTR)ntlm_cred;
+        ret = SEC_E_OK;
+        break;
+
+    case SECPKG_CRED_OUTBOUND:
+        auth_data = pAuthData;
+        if (!(ntlm_cred = heap_alloc(sizeof(*ntlm_cred)))) return SEC_E_INSUFFICIENT_MEMORY;
+
+        ntlm_cred->mode = NTLM_CLIENT;
+        ntlm_cred->username_arg = NULL;
+        ntlm_cred->domain_arg = NULL;
+        ntlm_cred->password = NULL;
+        ntlm_cred->pwlen = 0;
+        ntlm_cred->no_cached_credentials = 0;
+
+        if (pAuthData)
+        {
+            int domain_len = 0, user_len = 0, password_len = 0;
+
+            if (auth_data->Flags & SEC_WINNT_AUTH_IDENTITY_ANSI)
+            {
+                if (auth_data->DomainLength)
+                {
+                    domain_len = MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->Domain,
+                                                     auth_data->DomainLength, NULL, 0);
+                    if (!(domain = heap_alloc(sizeof(WCHAR) * domain_len))) goto done;
+                    MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->Domain, auth_data->DomainLength,
+                                        domain, domain_len);
+                }
+                if (auth_data->UserLength)
+                {
+                    user_len = MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->User,
+                                                   auth_data->UserLength, NULL, 0);
+                    if (!(user = heap_alloc(sizeof(WCHAR) * user_len))) goto done;
+                    MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->User, auth_data->UserLength,
+                                        user, user_len);
+                }
+                if (auth_data->PasswordLength)
+                {
+                    password_len = MultiByteToWideChar(CP_ACP, 0,(char *)auth_data->Password,
+                                                       auth_data->PasswordLength, NULL, 0);
+                    if (!(password = heap_alloc(sizeof(WCHAR) * password_len))) goto done;
+                    MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->Password, auth_data->PasswordLength,
+                                        password, password_len);
+                }
+            }
             else
             {
-                ntlm_cred->mode = NTLM_SERVER;
-                ntlm_cred->username_arg = NULL;
-                ntlm_cred->domain_arg = NULL;
-                ntlm_cred->password = NULL;
-                ntlm_cred->pwlen = 0;
-                ntlm_cred->no_cached_credentials = 0;
+                domain = auth_data->Domain;
+                domain_len = auth_data->DomainLength;
 
-                phCredential->dwUpper = fCredentialUse;
-                phCredential->dwLower = (ULONG_PTR)ntlm_cred;
-                ret = SEC_E_OK;
+                user = auth_data->User;
+                user_len = auth_data->UserLength;
+
+                password = auth_data->Password;
+                password_len = auth_data->PasswordLength;
             }
-            break;
-        case SECPKG_CRED_OUTBOUND:
+
+            TRACE("Username is %s\n", debugstr_wn(user, user_len));
+            TRACE("Domain name is %s\n", debugstr_wn(domain, domain_len));
+
+            ntlm_cred->username_arg = ntlm_GetUsernameArg(user, user_len);
+            ntlm_cred->domain_arg = ntlm_GetDomainArg(domain, domain_len);
+
+            if (password_len)
             {
-                auth_data = pAuthData;
-                ntlm_cred = HeapAlloc(GetProcessHeap(), 0, sizeof(*ntlm_cred));
-                if (!ntlm_cred)
-                {
-                    ret = SEC_E_INSUFFICIENT_MEMORY;
-                    break;
-                }
-                ntlm_cred->mode = NTLM_CLIENT;
-                ntlm_cred->username_arg = NULL;
-                ntlm_cred->domain_arg = NULL;
-                ntlm_cred->password = NULL;
-                ntlm_cred->pwlen = 0;
-                ntlm_cred->no_cached_credentials = 0;
-
-                if(pAuthData != NULL)
-                {
-                    int domain_len = 0, user_len = 0, password_len = 0;
-
-                    if (auth_data->Flags & SEC_WINNT_AUTH_IDENTITY_ANSI)
-                    {
-                        if (auth_data->DomainLength)
-                        {
-                            domain_len = MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->Domain,
-                                                             auth_data->DomainLength, NULL, 0);
-                            domain = HeapAlloc(GetProcessHeap(), 0, sizeof(WCHAR) * domain_len);
-                            if(!domain)
-                            {
-                                ret = SEC_E_INSUFFICIENT_MEMORY;
-                                break;
-                            }
-                            MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->Domain, auth_data->DomainLength,
-                                                domain, domain_len);
-                        }
-
-                        if (auth_data->UserLength)
-                        {
-                            user_len = MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->User,
-                                                           auth_data->UserLength, NULL, 0);
-                            user = HeapAlloc(GetProcessHeap(), 0, sizeof(WCHAR) * user_len);
-                            if(!user)
-                            {
-                                ret = SEC_E_INSUFFICIENT_MEMORY;
-                                break;
-                            }
-                            MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->User, auth_data->UserLength,
-                                                user, user_len);
-                        }
-
-                        if (auth_data->PasswordLength)
-                        {
-                            password_len = MultiByteToWideChar(CP_ACP, 0,(char *)auth_data->Password,
-                                                               auth_data->PasswordLength, NULL, 0);
-                            password = HeapAlloc(GetProcessHeap(), 0, sizeof(WCHAR) * password_len);
-                            if(!password)
-                            {
-                                ret = SEC_E_INSUFFICIENT_MEMORY;
-                                break;
-                            }
-                            MultiByteToWideChar(CP_ACP, 0, (char *)auth_data->Password, auth_data->PasswordLength,
-                                                password, password_len);
-                        }
-                    }
-                    else
-                    {
-                        domain = auth_data->Domain;
-                        domain_len = auth_data->DomainLength;
-
-                        user = auth_data->User;
-                        user_len = auth_data->UserLength;
-
-                        password = auth_data->Password;
-                        password_len = auth_data->PasswordLength;
-                    }
-
-                    TRACE("Username is %s\n", debugstr_wn(user, user_len));
-                    TRACE("Domain name is %s\n", debugstr_wn(domain, domain_len));
-
-                    ntlm_cred->username_arg = ntlm_GetUsernameArg(user, user_len);
-                    ntlm_cred->domain_arg = ntlm_GetDomainArg(domain, domain_len);
-
-                    if(password_len != 0)
-                    {
-                        ntlm_cred->pwlen = WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, password,
-                                                               password_len, NULL, 0, NULL, NULL);
-
-                        ntlm_cred->password = HeapAlloc(GetProcessHeap(), 0,
-                                                        ntlm_cred->pwlen);
-                        if(!ntlm_cred->password)
-                        {
-                            ret = SEC_E_INSUFFICIENT_MEMORY;
-                            break;
-                        }
-
-                        WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, password, password_len,
-                                            ntlm_cred->password, ntlm_cred->pwlen, NULL, NULL);
-                    }
-                }
-
-                phCredential->dwUpper = fCredentialUse;
-                phCredential->dwLower = (ULONG_PTR)ntlm_cred;
-                TRACE("ACH phCredential->dwUpper: 0x%08lx, dwLower: 0x%08lx\n",
-                      phCredential->dwUpper, phCredential->dwLower);
-                ret = SEC_E_OK;
-                break;
+                ntlm_cred->pwlen = WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, password,
+                                                       password_len, NULL, 0, NULL, NULL);
+                if (!(ntlm_cred->password = heap_alloc(ntlm_cred->pwlen))) goto done;
+                WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, password, password_len,
+                                    ntlm_cred->password, ntlm_cred->pwlen, NULL, NULL);
             }
-        case SECPKG_CRED_BOTH:
-            FIXME("AcquireCredentialsHandle: SECPKG_CRED_BOTH stub\n");
-            ret = SEC_E_UNSUPPORTED_FUNCTION;
-            phCredential = NULL;
-            break;
-        default:
-            phCredential = NULL;
-            ret = SEC_E_UNKNOWN_CREDENTIALS;
+        }
+
+        phCredential->dwUpper = fCredentialUse;
+        phCredential->dwLower = (ULONG_PTR)ntlm_cred;
+        TRACE("ACH phCredential->dwUpper: 0x%08lx, dwLower: 0x%08lx\n", phCredential->dwUpper,
+              phCredential->dwLower);
+        ret = SEC_E_OK;
+        break;
+
+    case SECPKG_CRED_BOTH:
+        FIXME("AcquireCredentialsHandle: SECPKG_CRED_BOTH stub\n");
+        ret = SEC_E_UNSUPPORTED_FUNCTION;
+        break;
+
+    default:
+        ret = SEC_E_UNKNOWN_CREDENTIALS;
     }
 
-    if (auth_data && auth_data->Flags & SEC_WINNT_AUTH_IDENTITY_ANSI)
+done:
+    if (auth_data && (auth_data->Flags & SEC_WINNT_AUTH_IDENTITY_ANSI))
     {
-        HeapFree(GetProcessHeap(), 0, domain);
-        HeapFree(GetProcessHeap(), 0, user);
-        HeapFree(GetProcessHeap(), 0, password);
+        heap_free(domain);
+        heap_free(user);
+        heap_free(password);
     }
+    if (ret != SEC_E_OK) heap_free( ntlm_cred );
+
     return ret;
 }
 
@@ -295,81 +261,54 @@ static SECURITY_STATUS SEC_ENTRY ntlm_AcquireCredentialsHandleA(
  PLUID pLogonID, PVOID pAuthData, SEC_GET_KEY_FN pGetKeyFn,
  PVOID pGetKeyArgument, PCredHandle phCredential, PTimeStamp ptsExpiry)
 {
-    SECURITY_STATUS ret;
+    SECURITY_STATUS ret = SEC_E_INSUFFICIENT_MEMORY;
     int user_sizeW, domain_sizeW, passwd_sizeW;
-    
     SEC_WCHAR *user = NULL, *domain = NULL, *passwd = NULL, *package = NULL;
-    
     PSEC_WINNT_AUTH_IDENTITY_W pAuthDataW = NULL;
-    PSEC_WINNT_AUTH_IDENTITY_A identity  = NULL;
+    PSEC_WINNT_AUTH_IDENTITY_A id  = NULL;
 
     TRACE("(%s, %s, 0x%08x, %p, %p, %p, %p, %p, %p)\n",
      debugstr_a(pszPrincipal), debugstr_a(pszPackage), fCredentialUse,
      pLogonID, pAuthData, pGetKeyFn, pGetKeyArgument, phCredential, ptsExpiry);
-    
-    if(pszPackage != NULL)
-    {
-        int package_sizeW = MultiByteToWideChar(CP_ACP, 0, pszPackage, -1,
-                NULL, 0);
 
-        package = HeapAlloc(GetProcessHeap(), 0, package_sizeW * 
-                sizeof(SEC_WCHAR));
+    if (pszPackage)
+    {
+        int package_sizeW = MultiByteToWideChar(CP_ACP, 0, pszPackage, -1, NULL, 0);
+        if (!(package = heap_alloc(package_sizeW * sizeof(SEC_WCHAR)))) return SEC_E_INSUFFICIENT_MEMORY;
         MultiByteToWideChar(CP_ACP, 0, pszPackage, -1, package, package_sizeW);
     }
 
-    
-    if(pAuthData != NULL)
+    if (pAuthData)
     {
-        identity = pAuthData;
-
-        if(identity->Flags == SEC_WINNT_AUTH_IDENTITY_ANSI)
+        id = pAuthData;
+        if (id->Flags == SEC_WINNT_AUTH_IDENTITY_ANSI)
         {
-            pAuthDataW = HeapAlloc(GetProcessHeap(), 0, 
-                    sizeof(SEC_WINNT_AUTH_IDENTITY_W));
+            if (!(pAuthDataW = heap_alloc(sizeof(SEC_WINNT_AUTH_IDENTITY_W)))) goto done;
 
-            if(identity->UserLength != 0)
-            {
-                user_sizeW = MultiByteToWideChar(CP_ACP, 0, 
-                    (LPCSTR)identity->User, identity->UserLength, NULL, 0);
-                user = HeapAlloc(GetProcessHeap(), 0, user_sizeW * 
-                        sizeof(SEC_WCHAR));
-                MultiByteToWideChar(CP_ACP, 0, (LPCSTR)identity->User, 
-                    identity->UserLength, user, user_sizeW);
-            }
+            if (!id->UserLength) user_sizeW = 0;
             else
             {
-                user_sizeW = 0;
-            }
-             
-            if(identity->DomainLength != 0)
-            {
-                domain_sizeW = MultiByteToWideChar(CP_ACP, 0, 
-                    (LPCSTR)identity->Domain, identity->DomainLength, NULL, 0);
-                domain = HeapAlloc(GetProcessHeap(), 0, domain_sizeW 
-                    * sizeof(SEC_WCHAR));
-                MultiByteToWideChar(CP_ACP, 0, (LPCSTR)identity->Domain, 
-                    identity->DomainLength, domain, domain_sizeW);
-            }
-            else
-            {
-                domain_sizeW = 0;
+                user_sizeW = MultiByteToWideChar(CP_ACP, 0, (LPCSTR)id->User, id->UserLength, NULL, 0);
+                if (!(user = heap_alloc(user_sizeW * sizeof(SEC_WCHAR)))) goto done;
+                MultiByteToWideChar(CP_ACP, 0, (LPCSTR)id->User, id->UserLength, user, user_sizeW);
             }
 
-            if(identity->PasswordLength != 0)
-            {
-                passwd_sizeW = MultiByteToWideChar(CP_ACP, 0, 
-                    (LPCSTR)identity->Password, identity->PasswordLength,
-                    NULL, 0);
-                passwd = HeapAlloc(GetProcessHeap(), 0, passwd_sizeW
-                    * sizeof(SEC_WCHAR));
-                MultiByteToWideChar(CP_ACP, 0, (LPCSTR)identity->Password,
-                    identity->PasswordLength, passwd, passwd_sizeW);
-            }
+            if (!id->DomainLength) domain_sizeW = 0;
             else
             {
-                passwd_sizeW = 0;
+                domain_sizeW = MultiByteToWideChar(CP_ACP, 0, (LPCSTR)id->Domain, id->DomainLength, NULL, 0);
+                if (!(domain = heap_alloc(domain_sizeW * sizeof(SEC_WCHAR)))) goto done;
+                MultiByteToWideChar(CP_ACP, 0, (LPCSTR)id->Domain, id->DomainLength, domain, domain_sizeW);
             }
-            
+
+            if (!id->PasswordLength) passwd_sizeW = 0;
+            else
+            {
+                passwd_sizeW = MultiByteToWideChar(CP_ACP, 0, (LPCSTR)id->Password, id->PasswordLength, NULL, 0);
+                if (!(passwd = heap_alloc(passwd_sizeW * sizeof(SEC_WCHAR)))) goto done;
+                MultiByteToWideChar(CP_ACP, 0, (LPCSTR)id->Password, id->PasswordLength, passwd, passwd_sizeW);
+            }
+
             pAuthDataW->Flags = SEC_WINNT_AUTH_IDENTITY_UNICODE;
             pAuthDataW->User = user;
             pAuthDataW->UserLength = user_sizeW;
@@ -378,23 +317,20 @@ static SECURITY_STATUS SEC_ENTRY ntlm_AcquireCredentialsHandleA(
             pAuthDataW->Password = passwd;
             pAuthDataW->PasswordLength = passwd_sizeW;
         }
-        else
-        {
-            pAuthDataW = (PSEC_WINNT_AUTH_IDENTITY_W)identity;
-        }
-    }       
-    
+        else pAuthDataW = (PSEC_WINNT_AUTH_IDENTITY_W)id;
+    }
+
     ret = ntlm_AcquireCredentialsHandleW(NULL, package, fCredentialUse, 
             pLogonID, pAuthDataW, pGetKeyFn, pGetKeyArgument, phCredential,
             ptsExpiry);
-    
-    HeapFree(GetProcessHeap(), 0, package);
-    HeapFree(GetProcessHeap(), 0, user);
-    HeapFree(GetProcessHeap(), 0, domain);
-    HeapFree(GetProcessHeap(), 0, passwd);
-    if(pAuthDataW != (PSEC_WINNT_AUTH_IDENTITY_W)identity)
-        HeapFree(GetProcessHeap(), 0, pAuthDataW);
-    
+
+done:
+    heap_free(package);
+    heap_free(user);
+    heap_free(domain);
+    heap_free(passwd);
+    if (pAuthDataW != (PSEC_WINNT_AUTH_IDENTITY_W)id) heap_free(pAuthDataW);
+
     return ret;
 }
 
@@ -468,23 +404,20 @@ static BOOL ntlm_GetCachedCredential(const SEC_WCHAR *pszTargetName, PCREDENTIAL
         p = pszHost + strlenW(pszHost);
     }
 
-    pszHostOnly = HeapAlloc(GetProcessHeap(), 0, (p - pszHost + 1) * sizeof(WCHAR));
-    if (!pszHostOnly)
-        return FALSE;
-
+    if (!(pszHostOnly = heap_alloc((p - pszHost + 1) * sizeof(WCHAR)))) return FALSE;
     memcpy(pszHostOnly, pszHost, (p - pszHost) * sizeof(WCHAR));
     pszHostOnly[p - pszHost] = '\0';
 
     ret = CredReadW(pszHostOnly, CRED_TYPE_DOMAIN_PASSWORD, 0, cred);
 
-    HeapFree(GetProcessHeap(), 0, pszHostOnly);
+    heap_free(pszHostOnly);
     return ret;
 }
 
 /***********************************************************************
  *              InitializeSecurityContextW
  */
-SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
+static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
  PCredHandle phCredential, PCtxtHandle phContext, SEC_WCHAR *pszTargetName, 
  ULONG fContextReq, ULONG Reserved1, ULONG TargetDataRep, 
  PSecBufferDesc pInput, ULONG Reserved2, PCtxtHandle phNewContext, 
@@ -524,8 +457,8 @@ SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
         TRACE("Setting SECURITY_NETWORK_DREP\n");
     }
 
-    buffer = HeapAlloc(GetProcessHeap(), 0, sizeof(char) * NTLM_MAX_BUF);
-    bin = HeapAlloc(GetProcessHeap(), 0, sizeof(BYTE) * NTLM_MAX_BUF);
+    buffer = heap_alloc(sizeof(char) * NTLM_MAX_BUF);
+    bin = heap_alloc(sizeof(BYTE) * NTLM_MAX_BUF);
 
     if((phContext == NULL) && (pInput == NULL))
     {
@@ -586,8 +519,7 @@ SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
                         cred->CredentialBlobSize / sizeof(WCHAR), NULL, 0,
                         NULL, NULL);
 
-                    password = HeapAlloc(GetProcessHeap(), 0, pwlen);
-
+                    password = heap_alloc(pwlen);
                     WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS,
                                         (LPWSTR)cred->CredentialBlob,
                                         cred->CredentialBlobSize / sizeof(WCHAR),
@@ -627,7 +559,7 @@ SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
             goto isc_end;
 
         helper->mode = NTLM_CLIENT;
-        helper->session_key = HeapAlloc(GetProcessHeap(), 0, 16);
+        helper->session_key = heap_alloc(16);
         if (!helper->session_key)
         {
             cleanup_helper(helper);
@@ -646,15 +578,13 @@ SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
                                               password ? password : ntlm_cred->password,
                                               password ? pwlen : ntlm_cred->pwlen,
                                               NULL, 0);
-            unicode_password = HeapAlloc(GetProcessHeap(), 0,
-                                         passwd_lenW * sizeof(SEC_WCHAR));
+            unicode_password = heap_alloc(passwd_lenW * sizeof(SEC_WCHAR));
             MultiByteToWideChar(CP_ACP, 0, password ? password : ntlm_cred->password,
                                 password ? pwlen : ntlm_cred->pwlen, unicode_password, passwd_lenW);
 
             SECUR32_CreateNTLM1SessionKey((PBYTE)unicode_password,
                                           passwd_lenW * sizeof(SEC_WCHAR), helper->session_key);
-
-            HeapFree(GetProcessHeap(), 0, unicode_password);
+            heap_free(unicode_password);
         }
         else
             memset(helper->session_key, 0, 16);
@@ -663,8 +593,7 @@ SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
          * "SF NTLMSSP_FEATURE_SIGN NTLMSSP_FEATURE_SEAL
          * NTLMSSP_FEATURE_SESSION_KEY"
          */
-        want_flags = HeapAlloc(GetProcessHeap(), 0, 73);
-        if(want_flags == NULL)
+        if (!(want_flags = heap_alloc(73)))
         {
             cleanup_helper(helper);
             ret = SEC_E_INSUFFICIENT_MEMORY;
@@ -891,7 +820,7 @@ SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
 
     if (fContextReq & ISC_REQ_ALLOCATE_MEMORY)
     {
-        pOutput->pBuffers[token_idx].pvBuffer = HeapAlloc(GetProcessHeap(), 0, bin_len);
+        pOutput->pBuffers[token_idx].pvBuffer = heap_alloc(bin_len);
         pOutput->pBuffers[token_idx].cbBuffer = bin_len;
     }
     else if (pOutput->pBuffers[token_idx].cbBuffer < bin_len)
@@ -957,9 +886,8 @@ SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
                 TRACE("Failed to decode session key\n");
             }
             TRACE("Session key is %s\n", debugstr_a(buffer+3));
-            HeapFree(GetProcessHeap(), 0, helper->session_key);
-            helper->session_key = HeapAlloc(GetProcessHeap(), 0, bin_len);
-            if(!helper->session_key)
+            heap_free(helper->session_key);
+            if (!(helper->session_key = heap_alloc(bin_len)))
             {
                 ret = SEC_E_INSUFFICIENT_MEMORY;
                 goto isc_end;
@@ -982,12 +910,12 @@ SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
     }
 
 isc_end:
-    HeapFree(GetProcessHeap(), 0, username);
-    HeapFree(GetProcessHeap(), 0, domain);
-    HeapFree(GetProcessHeap(), 0, password);
-    HeapFree(GetProcessHeap(), 0, want_flags);
-    HeapFree(GetProcessHeap(), 0, buffer);
-    HeapFree(GetProcessHeap(), 0, bin);
+    heap_free(username);
+    heap_free(domain);
+    heap_free(password);
+    heap_free(want_flags);
+    heap_free(buffer);
+    heap_free(bin);
     return ret;
 }
 
@@ -1007,28 +935,25 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextA(
      debugstr_a(pszTargetName), fContextReq, Reserved1, TargetDataRep, pInput,
      Reserved1, phNewContext, pOutput, pfContextAttr, ptsExpiry);
 
-    if(pszTargetName != NULL)
+    if (pszTargetName)
     {
-        int target_size = MultiByteToWideChar(CP_ACP, 0, pszTargetName,
-            strlen(pszTargetName)+1, NULL, 0);
-        target = HeapAlloc(GetProcessHeap(), 0, target_size *
-                sizeof(SEC_WCHAR));
-        MultiByteToWideChar(CP_ACP, 0, pszTargetName, strlen(pszTargetName)+1,
-            target, target_size);
+        int target_size = MultiByteToWideChar(CP_ACP, 0, pszTargetName, -1, NULL, 0);
+        if (!(target = heap_alloc(target_size * sizeof(SEC_WCHAR)))) return SEC_E_INSUFFICIENT_MEMORY;
+        MultiByteToWideChar(CP_ACP, 0, pszTargetName, -1, target, target_size);
     }
 
     ret = ntlm_InitializeSecurityContextW(phCredential, phContext, target,
             fContextReq, Reserved1, TargetDataRep, pInput, Reserved2,
             phNewContext, pOutput, pfContextAttr, ptsExpiry);
 
-    HeapFree(GetProcessHeap(), 0, target);
+    heap_free(target);
     return ret;
 }
 
 /***********************************************************************
  *              AcceptSecurityContext
  */
-SECURITY_STATUS SEC_ENTRY ntlm_AcceptSecurityContext(
+static SECURITY_STATUS SEC_ENTRY ntlm_AcceptSecurityContext(
  PCredHandle phCredential, PCtxtHandle phContext, PSecBufferDesc pInput,
  ULONG fContextReq, ULONG TargetDataRep, PCtxtHandle phNewContext, 
  PSecBufferDesc pOutput, ULONG *pfContextAttr, PTimeStamp ptsExpiry)
@@ -1045,8 +970,8 @@ SECURITY_STATUS SEC_ENTRY ntlm_AcceptSecurityContext(
      fContextReq, TargetDataRep, phNewContext, pOutput, pfContextAttr,
      ptsExpiry);
 
-    buffer = HeapAlloc(GetProcessHeap(), 0, sizeof(char) * NTLM_MAX_BUF);
-    bin    = HeapAlloc(GetProcessHeap(),0, sizeof(BYTE) * NTLM_MAX_BUF);
+    buffer = heap_alloc(sizeof(char) * NTLM_MAX_BUF);
+    bin    = heap_alloc(sizeof(BYTE) * NTLM_MAX_BUF);
 
     if(TargetDataRep == SECURITY_NETWORK_DREP){
         TRACE("Using SECURITY_NETWORK_DREP\n");
@@ -1104,8 +1029,7 @@ SECURITY_STATUS SEC_ENTRY ntlm_AcceptSecurityContext(
         helper->mode = NTLM_SERVER;
 
         /* Handle all the flags */
-        want_flags = HeapAlloc(GetProcessHeap(), 0, 73);
-        if(want_flags == NULL)
+        if (!(want_flags = heap_alloc(73)))
         {
             TRACE("Failed to allocate memory for the want_flags!\n");
             ret = SEC_E_INSUFFICIENT_MEMORY;
@@ -1338,9 +1262,8 @@ SECURITY_STATUS SEC_ENTRY ntlm_AcceptSecurityContext(
             if(strncmp(buffer, "BH ", 3) == 0)
             {
                 TRACE("Helper sent %s\n", debugstr_a(buffer+3));
-                HeapFree(GetProcessHeap(), 0, helper->session_key);
-                helper->session_key = HeapAlloc(GetProcessHeap(), 0, 16);
-                if (!helper->session_key)
+                heap_free(helper->session_key);
+                if (!(helper->session_key = heap_alloc(16)))
                 {
                     ret = SEC_E_INSUFFICIENT_MEMORY;
                     goto asc_end;
@@ -1356,9 +1279,8 @@ SECURITY_STATUS SEC_ENTRY ntlm_AcceptSecurityContext(
                     TRACE("Failed to decode session key\n");
                 }
                 TRACE("Session key is %s\n", debugstr_a(buffer+3));
-                HeapFree(GetProcessHeap(), 0, helper->session_key);
-                helper->session_key = HeapAlloc(GetProcessHeap(), 0, 16);
-                if(!helper->session_key)
+                heap_free(helper->session_key);
+                if (!(helper->session_key = heap_alloc(16)))
                 {
                     ret = SEC_E_INSUFFICIENT_MEMORY;
                     goto asc_end;
@@ -1375,9 +1297,9 @@ SECURITY_STATUS SEC_ENTRY ntlm_AcceptSecurityContext(
     phNewContext->dwLower = (ULONG_PTR)helper;
 
 asc_end:
-    HeapFree(GetProcessHeap(), 0, want_flags);
-    HeapFree(GetProcessHeap(), 0, buffer);
-    HeapFree(GetProcessHeap(), 0, bin);
+    heap_free(want_flags);
+    heap_free(buffer);
+    heap_free(bin);
     return ret;
 }
 
@@ -1398,7 +1320,7 @@ static SECURITY_STATUS SEC_ENTRY ntlm_CompleteAuthToken(PCtxtHandle phContext,
 /***********************************************************************
  *              DeleteSecurityContext
  */
-SECURITY_STATUS SEC_ENTRY ntlm_DeleteSecurityContext(PCtxtHandle phContext)
+static SECURITY_STATUS SEC_ENTRY ntlm_DeleteSecurityContext(PCtxtHandle phContext)
 {
     PNegoHelper helper;
 
@@ -1414,20 +1336,101 @@ SECURITY_STATUS SEC_ENTRY ntlm_DeleteSecurityContext(PCtxtHandle phContext)
     SECUR32_arc4Cleanup(helper->crypt.ntlm.a4i);
     SECUR32_arc4Cleanup(helper->crypt.ntlm2.send_a4i);
     SECUR32_arc4Cleanup(helper->crypt.ntlm2.recv_a4i);
-    HeapFree(GetProcessHeap(), 0, helper->crypt.ntlm2.send_sign_key);
-    HeapFree(GetProcessHeap(), 0, helper->crypt.ntlm2.send_seal_key);
-    HeapFree(GetProcessHeap(), 0, helper->crypt.ntlm2.recv_sign_key);
-    HeapFree(GetProcessHeap(), 0, helper->crypt.ntlm2.recv_seal_key);
+    heap_free(helper->crypt.ntlm2.send_sign_key);
+    heap_free(helper->crypt.ntlm2.send_seal_key);
+    heap_free(helper->crypt.ntlm2.recv_sign_key);
+    heap_free(helper->crypt.ntlm2.recv_seal_key);
 
     cleanup_helper(helper);
 
     return SEC_E_OK;
 }
 
+#define NTLM_COMMENT \
+   { 'N', 'T', 'L', 'M', ' ', \
+     'S', 'e', 'c', 'u', 'r', 'i', 't', 'y', ' ', \
+     'P', 'a', 'c', 'k', 'a', 'g', 'e', 0}
+
+static CHAR ntlm_comment_A[] = NTLM_COMMENT;
+static WCHAR ntlm_comment_W[] = NTLM_COMMENT;
+
+#define NTLM_NAME {'N', 'T', 'L', 'M', 0}
+
+static char ntlm_name_A[] = NTLM_NAME;
+static WCHAR ntlm_name_W[] = NTLM_NAME;
+
+#define NTLM_CAPS ( \
+    SECPKG_FLAG_INTEGRITY  | \
+    SECPKG_FLAG_PRIVACY    | \
+    SECPKG_FLAG_TOKEN_ONLY | \
+    SECPKG_FLAG_CONNECTION | \
+    SECPKG_FLAG_MULTI_REQUIRED    | \
+    SECPKG_FLAG_IMPERSONATION     | \
+    SECPKG_FLAG_ACCEPT_WIN32_NAME | \
+    SECPKG_FLAG_NEGOTIABLE        | \
+    SECPKG_FLAG_LOGON             | \
+    SECPKG_FLAG_RESTRICTED_TOKENS )
+
+static const SecPkgInfoW infoW = {
+    NTLM_CAPS,
+    1,
+    RPC_C_AUTHN_WINNT,
+    NTLM_MAX_BUF,
+    ntlm_name_W,
+    ntlm_comment_W
+};
+
+static const SecPkgInfoA infoA = {
+    NTLM_CAPS,
+    1,
+    RPC_C_AUTHN_WINNT,
+    NTLM_MAX_BUF,
+    ntlm_name_A,
+    ntlm_comment_A
+};
+
+SecPkgInfoA *ntlm_package_infoA = (SecPkgInfoA *)&infoA;
+SecPkgInfoW *ntlm_package_infoW = (SecPkgInfoW *)&infoW;
+
+static SecPkgInfoW *build_package_infoW( const SecPkgInfoW *info )
+{
+    SecPkgInfoW *ret;
+    DWORD size_name = (strlenW(info->Name) + 1) * sizeof(WCHAR);
+    DWORD size_comment = (strlenW(info->Comment) + 1) * sizeof(WCHAR);
+
+    if (!(ret = heap_alloc( sizeof(*ret) + size_name + size_comment ))) return NULL;
+    ret->fCapabilities = info->fCapabilities;
+    ret->wVersion      = info->wVersion;
+    ret->wRPCID        = info->wRPCID;
+    ret->cbMaxToken    = info->cbMaxToken;
+    ret->Name          = (SEC_WCHAR *)(ret + 1);
+    memcpy( ret->Name, info->Name, size_name );
+    ret->Comment       = (SEC_WCHAR *)((char *)ret->Name + size_name);
+    memcpy( ret->Comment, info->Comment, size_comment );
+    return ret;
+}
+
+static SecPkgInfoA *build_package_infoA( const SecPkgInfoA *info )
+{
+    SecPkgInfoA *ret;
+    DWORD size_name = strlen(info->Name) + 1, size_comment = strlen(info->Comment) + 1;
+
+    if (!(ret = heap_alloc( sizeof(*ret) + size_name + size_comment ))) return NULL;
+    ret->fCapabilities = info->fCapabilities;
+    ret->wVersion      = info->wVersion;
+    ret->wRPCID        = info->wRPCID;
+    ret->cbMaxToken    = info->cbMaxToken;
+    ret->Name          = (SEC_CHAR *)(ret + 1);
+    memcpy( ret->Name, info->Name, size_name );
+    ret->Comment       = ret->Name + size_name;
+    memcpy( ret->Comment, info->Comment, size_comment );
+    return ret;
+}
+
 /***********************************************************************
  *              QueryContextAttributesW
  */
-SECURITY_STATUS SEC_ENTRY ntlm_QueryContextAttributesW(PCtxtHandle phContext,
+static SECURITY_STATUS SEC_ENTRY ntlm_QueryContextAttributesW(PCtxtHandle phContext,
  ULONG ulAttribute, void *pBuffer)
 {
     TRACE("%p %d %p\n", phContext, ulAttribute, pBuffer);
@@ -1456,7 +1459,13 @@ SECURITY_STATUS SEC_ENTRY ntlm_QueryContextAttributesW(PCtxtHandle phContext,
         _x(SECPKG_ATTR_LIFESPAN);
         _x(SECPKG_ATTR_NAMES);
         _x(SECPKG_ATTR_NATIVE_NAMES);
-        _x(SECPKG_ATTR_NEGOTIATION_INFO);
+        case SECPKG_ATTR_NEGOTIATION_INFO:
+        {
+            SecPkgContext_NegotiationInfoW *info = (SecPkgContext_NegotiationInfoW *)pBuffer;
+            if (!(info->PackageInfo = build_package_infoW( &infoW ))) return SEC_E_INSUFFICIENT_MEMORY;
+            info->NegotiationState = SECPKG_NEGOTIATION_COMPLETE;
+            return SEC_E_OK;
+        }
         _x(SECPKG_ATTR_PACKAGE_INFO);
         _x(SECPKG_ATTR_PASSWORD_EXPIRY);
         _x(SECPKG_ATTR_SESSION_KEY);
@@ -1482,10 +1491,21 @@ SECURITY_STATUS SEC_ENTRY ntlm_QueryContextAttributesW(PCtxtHandle phContext,
 /***********************************************************************
  *              QueryContextAttributesA
  */
-SECURITY_STATUS SEC_ENTRY ntlm_QueryContextAttributesA(PCtxtHandle phContext,
+static SECURITY_STATUS SEC_ENTRY ntlm_QueryContextAttributesA(PCtxtHandle phContext,
  ULONG ulAttribute, void *pBuffer)
 {
-    return ntlm_QueryContextAttributesW(phContext, ulAttribute, pBuffer);
+    switch(ulAttribute)
+    {
+    case SECPKG_ATTR_NEGOTIATION_INFO:
+    {
+        SecPkgContext_NegotiationInfoA *info = (SecPkgContext_NegotiationInfoA *)pBuffer;
+        if (!(info->PackageInfo = build_package_infoA( &infoA ))) return SEC_E_INSUFFICIENT_MEMORY;
+        info->NegotiationState = SECPKG_NEGOTIATION_COMPLETE;
+        return SEC_E_OK;
+    }
+    default:
+        return ntlm_QueryContextAttributesW( phContext, ulAttribute, pBuffer );
+    }
 }
 
 /***********************************************************************
@@ -1662,7 +1682,7 @@ static SECURITY_STATUS ntlm_CreateSignature(PNegoHelper helper, PSecBufferDesc p
 /***********************************************************************
  *              MakeSignature
  */
-SECURITY_STATUS SEC_ENTRY ntlm_MakeSignature(PCtxtHandle phContext,
+static SECURITY_STATUS SEC_ENTRY ntlm_MakeSignature(PCtxtHandle phContext,
     ULONG fQOP, PSecBufferDesc pMessage, ULONG MessageSeqNo)
 {
     PNegoHelper helper;
@@ -1697,7 +1717,7 @@ SECURITY_STATUS SEC_ENTRY ntlm_MakeSignature(PCtxtHandle phContext,
 /***********************************************************************
  *              VerifySignature
  */
-SECURITY_STATUS SEC_ENTRY ntlm_VerifySignature(PCtxtHandle phContext,
+static SECURITY_STATUS SEC_ENTRY ntlm_VerifySignature(PCtxtHandle phContext,
     PSecBufferDesc pMessage, ULONG MessageSeqNo, PULONG pfQOP)
 {
     PNegoHelper helper;
@@ -1727,7 +1747,7 @@ SECURITY_STATUS SEC_ENTRY ntlm_VerifySignature(PCtxtHandle phContext,
     helper = (PNegoHelper)phContext->dwLower;
     TRACE("Negotiated flags: 0x%08x\n", helper->neg_flags);
 
-    local_buff = HeapAlloc(GetProcessHeap(), 0, pMessage->cBuffers * sizeof(SecBuffer));
+    local_buff = heap_alloc(pMessage->cBuffers * sizeof(SecBuffer));
 
     local_desc.ulVersion = SECBUFFER_VERSION;
     local_desc.cBuffers = pMessage->cBuffers;
@@ -1758,8 +1778,7 @@ SECURITY_STATUS SEC_ENTRY ntlm_VerifySignature(PCtxtHandle phContext,
     else
         ret = SEC_E_OK;
 
-    HeapFree(GetProcessHeap(), 0, local_buff);
-
+    heap_free(local_buff);
     return ret;
 
 }
@@ -1767,32 +1786,27 @@ SECURITY_STATUS SEC_ENTRY ntlm_VerifySignature(PCtxtHandle phContext,
 /***********************************************************************
  *             FreeCredentialsHandle
  */
-SECURITY_STATUS SEC_ENTRY ntlm_FreeCredentialsHandle(PCredHandle phCredential)
+static SECURITY_STATUS SEC_ENTRY ntlm_FreeCredentialsHandle(PCredHandle phCredential)
 {
-    SECURITY_STATUS ret;
-
-    if(phCredential){
+    if (phCredential)
+    {
         PNtlmCredentials ntlm_cred = (PNtlmCredentials) phCredential->dwLower;
         phCredential->dwUpper = 0;
         phCredential->dwLower = 0;
-        if (ntlm_cred->password)
-            memset(ntlm_cred->password, 0, ntlm_cred->pwlen);
-        HeapFree(GetProcessHeap(), 0, ntlm_cred->password);
-        HeapFree(GetProcessHeap(), 0, ntlm_cred->username_arg);
-        HeapFree(GetProcessHeap(), 0, ntlm_cred->domain_arg);
-        HeapFree(GetProcessHeap(), 0, ntlm_cred);
-        ret = SEC_E_OK;
+        if (ntlm_cred->password) memset(ntlm_cred->password, 0, ntlm_cred->pwlen);
+        heap_free(ntlm_cred->password);
+        heap_free(ntlm_cred->username_arg);
+        heap_free(ntlm_cred->domain_arg);
+        heap_free(ntlm_cred);
     }
-    else
-        ret = SEC_E_OK;
-    
-    return ret;
+
+    return SEC_E_OK;
 }
 
 /***********************************************************************
  *             EncryptMessage
  */
-SECURITY_STATUS SEC_ENTRY ntlm_EncryptMessage(PCtxtHandle phContext,
+static SECURITY_STATUS SEC_ENTRY ntlm_EncryptMessage(PCtxtHandle phContext,
     ULONG fQOP, PSecBufferDesc pMessage, ULONG MessageSeqNo)
 {
     PNegoHelper helper;
@@ -1863,7 +1877,7 @@ SECURITY_STATUS SEC_ENTRY ntlm_EncryptMessage(PCtxtHandle phContext,
 /***********************************************************************
  *             DecryptMessage
  */
-SECURITY_STATUS SEC_ENTRY ntlm_DecryptMessage(PCtxtHandle phContext,
+static SECURITY_STATUS SEC_ENTRY ntlm_DecryptMessage(PCtxtHandle phContext,
     PSecBufferDesc pMessage, ULONG MessageSeqNo, PULONG pfQOP)
 {
     SECURITY_STATUS ret;
@@ -1979,53 +1993,6 @@ static const SecurityFunctionTableW ntlmTableW = {
     NULL,   /* SetContextAttributesW */
 };
 
-#define NTLM_COMMENT \
-   { 'N', 'T', 'L', 'M', ' ', \
-     'S', 'e', 'c', 'u', 'r', 'i', 't', 'y', ' ', \
-     'P', 'a', 'c', 'k', 'a', 'g', 'e', 0}
-
-static CHAR ntlm_comment_A[] = NTLM_COMMENT;
-static WCHAR ntlm_comment_W[] = NTLM_COMMENT;
-
-#define NTLM_NAME {'N', 'T', 'L', 'M', 0}
-
-static char ntlm_name_A[] = NTLM_NAME;
-static WCHAR ntlm_name_W[] = NTLM_NAME;
-
-/* According to Windows, NTLM has the following capabilities.  */
-#define CAPS ( \
-    SECPKG_FLAG_INTEGRITY  | \
-    SECPKG_FLAG_PRIVACY    | \
-    SECPKG_FLAG_TOKEN_ONLY | \
-    SECPKG_FLAG_CONNECTION | \
-    SECPKG_FLAG_MULTI_REQUIRED    | \
-    SECPKG_FLAG_IMPERSONATION     | \
-    SECPKG_FLAG_ACCEPT_WIN32_NAME | \
-    SECPKG_FLAG_NEGOTIABLE        | \
-    SECPKG_FLAG_LOGON             | \
-    SECPKG_FLAG_RESTRICTED_TOKENS )
-
-static const SecPkgInfoW infoW = {
-    CAPS,
-    1,
-    RPC_C_AUTHN_WINNT,
-    NTLM_MAX_BUF,
-    ntlm_name_W,
-    ntlm_comment_W
-};
-
-static const SecPkgInfoA infoA = {
-    CAPS,
-    1,
-    RPC_C_AUTHN_WINNT,
-    NTLM_MAX_BUF,
-    ntlm_name_A,
-    ntlm_comment_A
-};
-
-SecPkgInfoA *ntlm_package_infoA = (SecPkgInfoA *)&infoA;
-SecPkgInfoW *ntlm_package_infoW = (SecPkgInfoW *)&infoW;
-
 void SECUR32_initNTLMSP(void)
 {
     PNegoHelper helper;
