@@ -233,42 +233,44 @@ static WPARAM map_wparam_char_WtoA( WPARAM wParam, DWORD len )
 /* call a 32-bit window procedure */
 static LRESULT call_window_proc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp, LRESULT *result, void *arg )
 {
+    DPI_AWARENESS_CONTEXT context;
     WNDPROC proc = arg;
 
     USER_CheckNotLock();
 
     hwnd = WIN_GetFullHandle( hwnd );
-    if (TRACE_ON(relay))
-        DPRINTF( "%04x:Call window proc %p (hwnd=%p,msg=%s,wp=%08lx,lp=%08lx)\n",
-                 GetCurrentThreadId(), proc, hwnd, SPY_GetMsgName(msg, hwnd), wp, lp );
+    TRACE_(relay)( "\1Call window proc %p (hwnd=%p,msg=%s,wp=%08lx,lp=%08lx)\n",
+                   proc, hwnd, SPY_GetMsgName(msg, hwnd), wp, lp );
 
+    context = SetThreadDpiAwarenessContext( GetWindowDpiAwarenessContext( hwnd ));
     *result = WINPROC_wrapper( proc, hwnd, msg, wp, lp );
+    SetThreadDpiAwarenessContext( context );
 
-    if (TRACE_ON(relay))
-        DPRINTF( "%04x:Ret  window proc %p (hwnd=%p,msg=%s,wp=%08lx,lp=%08lx) retval=%08lx\n",
-                 GetCurrentThreadId(), proc, hwnd, SPY_GetMsgName(msg, hwnd), wp, lp, *result );
+    TRACE_(relay)( "\1Ret  window proc %p (hwnd=%p,msg=%s,wp=%08lx,lp=%08lx) retval=%08lx\n",
+                   proc, hwnd, SPY_GetMsgName(msg, hwnd), wp, lp, *result );
     return *result;
 }
 
 /* call a 32-bit dialog procedure */
 static LRESULT call_dialog_proc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp, LRESULT *result, void *arg )
 {
+    DPI_AWARENESS_CONTEXT context;
     WNDPROC proc = arg;
     LRESULT ret;
 
     USER_CheckNotLock();
 
     hwnd = WIN_GetFullHandle( hwnd );
-    if (TRACE_ON(relay))
-        DPRINTF( "%04x:Call dialog proc %p (hwnd=%p,msg=%s,wp=%08lx,lp=%08lx)\n",
-                 GetCurrentThreadId(), proc, hwnd, SPY_GetMsgName(msg, hwnd), wp, lp );
+    TRACE_(relay)( "\1Call dialog proc %p (hwnd=%p,msg=%s,wp=%08lx,lp=%08lx)\n",
+                   proc, hwnd, SPY_GetMsgName(msg, hwnd), wp, lp );
 
+    context = SetThreadDpiAwarenessContext( GetWindowDpiAwarenessContext( hwnd ));
     ret = WINPROC_wrapper( proc, hwnd, msg, wp, lp );
     *result = GetWindowLongPtrW( hwnd, DWLP_MSGRESULT );
+    SetThreadDpiAwarenessContext( context );
 
-    if (TRACE_ON(relay))
-        DPRINTF( "%04x:Ret  dialog proc %p (hwnd=%p,msg=%s,wp=%08lx,lp=%08lx) retval=%08lx result=%08lx\n",
-                 GetCurrentThreadId(), proc, hwnd, SPY_GetMsgName(msg, hwnd), wp, lp, ret, *result );
+    TRACE_(relay)( "\1Ret  dialog proc %p (hwnd=%p,msg=%s,wp=%08lx,lp=%08lx) retval=%08lx result=%08lx\n",
+                   proc, hwnd, SPY_GetMsgName(msg, hwnd), wp, lp, ret, *result );
     return ret;
 }
 
@@ -460,7 +462,7 @@ LRESULT WINPROC_CallProcAtoW( winproc_callback_t callback, HWND hwnd, UINT msg,
             {
                 len = 0;
                 if (*result)
-                    RtlUnicodeToMultiByteN( str, wParam - 1, &len, ptr, strlenW(ptr) * sizeof(WCHAR) );
+                    RtlUnicodeToMultiByteN( str, wParam - 1, &len, ptr, ret * sizeof(WCHAR) );
                 str[len] = 0;
                 *result = len;
             }
@@ -682,7 +684,7 @@ static LRESULT WINPROC_CallProcWtoA( winproc_callback_t callback, HWND hwnd, UIN
             {
                 if (*result)
                 {
-                    RtlMultiByteToUnicodeN( (LPWSTR)lParam, wParam*sizeof(WCHAR), &len, ptr, strlen(ptr)+1 );
+                    RtlMultiByteToUnicodeN( (LPWSTR)lParam, wParam*sizeof(WCHAR), &len, ptr, ret + 1 );
                     *result = len/sizeof(WCHAR) - 1;  /* do not count terminating null */
                 }
                 ((LPWSTR)lParam)[*result] = 0;
@@ -873,10 +875,10 @@ BOOL WINPROC_call_window( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam,
                           LRESULT *result, BOOL unicode, enum wm_char_mapping mapping )
 {
     struct user_thread_info *thread_info = get_user_thread_info();
+    BOOL unicode_win, is_dialog;
     WND *wndPtr;
     WNDPROC func;
     WINDOWPROC *proc;
-    BOOL unicode_win;
 
     if (!(wndPtr = WIN_GetPtr( hwnd ))) return FALSE;
     if (wndPtr == WND_OTHER_PROCESS || wndPtr == WND_DESKTOP) return FALSE;
@@ -888,6 +890,7 @@ BOOL WINPROC_call_window( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam,
     func = wndPtr->winproc;
     proc = handle_to_proc( wndPtr->winproc );
     unicode_win = wndPtr->flags & WIN_ISUNICODE;
+    is_dialog = wndPtr->dlgInfo != NULL;
     WIN_ReleasePtr( wndPtr );
 
     if (thread_info->recursion_count > MAX_WINPROC_RECURSION) return FALSE;
@@ -897,6 +900,23 @@ BOOL WINPROC_call_window( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam,
     {
         if (proc == WINPROC_PROC16)
             WINPROC_CallProcWtoA( wow_handlers.call_window_proc, hwnd, msg, wParam, lParam, result, func );
+        else if (is_dialog)
+        {
+            if (unicode_win)
+            {
+                if (proc && proc->procW)
+                    call_window_proc( hwnd, msg, wParam, lParam, result, proc->procW );
+                else
+                    call_window_proc( hwnd, msg, wParam, lParam, result, func );
+            }
+            else
+            {
+                if (proc && proc->procA)
+                    WINPROC_CallProcWtoA( call_window_proc, hwnd, msg, wParam, lParam, result, proc->procA );
+                else
+                    WINPROC_CallProcWtoA( call_window_proc, hwnd, msg, wParam, lParam, result, func );
+            }
+        }
         else if (proc && proc->procW)
             call_window_proc( hwnd, msg, wParam, lParam, result, proc->procW );
         else if (proc)
@@ -910,6 +930,23 @@ BOOL WINPROC_call_window( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam,
     {
         if (proc == WINPROC_PROC16)
             wow_handlers.call_window_proc( hwnd, msg, wParam, lParam, result, func );
+        else if (is_dialog)
+        {
+            if (unicode_win)
+            {
+                if (proc && proc->procW)
+                    WINPROC_CallProcAtoW( call_window_proc, hwnd, msg, wParam, lParam, result, proc->procW, mapping );
+                else
+                    WINPROC_CallProcAtoW( call_window_proc, hwnd, msg, wParam, lParam, result, func, mapping );
+            }
+            else
+            {
+                if (proc && proc->procA)
+                    call_window_proc( hwnd, msg, wParam, lParam, result, proc->procA );
+                else
+                    call_window_proc( hwnd, msg, wParam, lParam, result, func );
+            }
+        }
         else if (proc && proc->procA)
             call_window_proc( hwnd, msg, wParam, lParam, result, proc->procA );
         else if (proc)
@@ -1016,14 +1053,9 @@ INT_PTR WINPROC_CallDlgProcA( DLGPROC func, HWND hwnd, UINT msg, WPARAM wParam,
         ret = wow_handlers.call_dialog_proc( hwnd, msg, wParam, lParam, &result, func );
         SetWindowLongPtrW( hwnd, DWLP_MSGRESULT, result );
     }
-    else if (proc->procW)
-    {
-        ret = WINPROC_CallProcAtoW( call_dialog_proc, hwnd, msg, wParam, lParam, &result,
-                                    proc->procW, WMCHAR_MAP_CALLWINDOWPROC );
-        SetWindowLongPtrW( hwnd, DWLP_MSGRESULT, result );
-    }
     else
-        ret = call_dialog_proc( hwnd, msg, wParam, lParam, &result, proc->procA );
+        ret = call_dialog_proc( hwnd, msg, wParam, lParam, &result, proc->procW ? proc->procW : proc->procA );
+
     return ret;
 }
 
@@ -1046,13 +1078,9 @@ INT_PTR WINPROC_CallDlgProcW( DLGPROC func, HWND hwnd, UINT msg, WPARAM wParam,
         ret = WINPROC_CallProcWtoA( wow_handlers.call_dialog_proc, hwnd, msg, wParam, lParam, &result, func );
         SetWindowLongPtrW( hwnd, DWLP_MSGRESULT, result );
     }
-    else if (proc->procA)
-    {
-        ret = WINPROC_CallProcWtoA( call_dialog_proc, hwnd, msg, wParam, lParam, &result, proc->procA );
-        SetWindowLongPtrW( hwnd, DWLP_MSGRESULT, result );
-    }
     else
-        ret = call_dialog_proc( hwnd, msg, wParam, lParam, &result, proc->procW );
+        ret = call_dialog_proc( hwnd, msg, wParam, lParam, &result, proc->procW ? proc->procW : proc->procA );
+
     return ret;
 }
 
