@@ -529,21 +529,27 @@ static HRESULT tiff_get_decode_info(TIFF *tiff, tiff_decode_info *decode_info)
 
     decode_info->resolution_unit = 0;
     pTIFFGetField(tiff, TIFFTAG_RESOLUTIONUNIT, &decode_info->resolution_unit);
-    if (decode_info->resolution_unit != 0)
-    {
-        ret = pTIFFGetField(tiff, TIFFTAG_XRESOLUTION, &decode_info->xres);
-        if (!ret)
-        {
-            WARN("missing X resolution\n");
-            decode_info->resolution_unit = 0;
-        }
 
-        ret = pTIFFGetField(tiff, TIFFTAG_YRESOLUTION, &decode_info->yres);
-        if (!ret)
-        {
-            WARN("missing Y resolution\n");
-            decode_info->resolution_unit = 0;
-        }
+    ret = pTIFFGetField(tiff, TIFFTAG_XRESOLUTION, &decode_info->xres);
+    if (!ret)
+    {
+        WARN("missing X resolution\n");
+    }
+    /* Emulate the behavior of current libtiff versions (libtiff commit a39f6131)
+     * yielding 0 instead of INFINITY for IFD_RATIONAL fields with denominator 0. */
+    if (!isfinite(decode_info->xres))
+    {
+        decode_info->xres = 0.0;
+    }
+
+    ret = pTIFFGetField(tiff, TIFFTAG_YRESOLUTION, &decode_info->yres);
+    if (!ret)
+    {
+        WARN("missing Y resolution\n");
+    }
+    if (!isfinite(decode_info->yres))
+    {
+        decode_info->yres = 0.0;
     }
 
     return S_OK;
@@ -666,27 +672,16 @@ static HRESULT WINAPI TiffDecoder_GetContainerFormat(IWICBitmapDecoder *iface,
 static HRESULT WINAPI TiffDecoder_GetDecoderInfo(IWICBitmapDecoder *iface,
     IWICBitmapDecoderInfo **ppIDecoderInfo)
 {
-    HRESULT hr;
-    IWICComponentInfo *compinfo;
-
     TRACE("(%p,%p)\n", iface, ppIDecoderInfo);
 
-    hr = CreateComponentInfo(&CLSID_WICTiffDecoder, &compinfo);
-    if (FAILED(hr)) return hr;
-
-    hr = IWICComponentInfo_QueryInterface(compinfo, &IID_IWICBitmapDecoderInfo,
-        (void**)ppIDecoderInfo);
-
-    IWICComponentInfo_Release(compinfo);
-
-    return hr;
+    return get_decoder_info(&CLSID_WICTiffDecoder, ppIDecoderInfo);
 }
 
 static HRESULT WINAPI TiffDecoder_CopyPalette(IWICBitmapDecoder *iface,
-    IWICPalette *pIPalette)
+    IWICPalette *palette)
 {
-    FIXME("(%p,%p): stub\n", iface, pIPalette);
-    return E_NOTIMPL;
+    TRACE("(%p,%p)\n", iface, palette);
+    return WINCODEC_ERR_PALETTEUNAVAILABLE;
 }
 
 static HRESULT WINAPI TiffDecoder_GetMetadataQueryReader(IWICBitmapDecoder *iface,
@@ -893,26 +888,28 @@ static HRESULT WINAPI TiffFrameDecode_GetResolution(IWICBitmapFrameDecode *iface
 {
     TiffFrameDecode *This = impl_from_IWICBitmapFrameDecode(iface);
 
-    switch (This->decode_info.resolution_unit)
+    if (This->decode_info.xres == 0 || This->decode_info.yres == 0)
     {
-    default:
-        FIXME("unknown resolution unit %i\n", This->decode_info.resolution_unit);
-        /* fall through */
-    case 0: /* Not set */
         *pDpiX = *pDpiY = 96.0;
-        break;
-    case 1: /* Relative measurements */
-        *pDpiX = 96.0;
-        *pDpiY = 96.0 * This->decode_info.yres / This->decode_info.xres;
-        break;
-    case 2: /* Inch */
-        *pDpiX = This->decode_info.xres;
-        *pDpiY = This->decode_info.yres;
-        break;
-    case 3: /* Centimeter */
-        *pDpiX = This->decode_info.xres / 2.54;
-        *pDpiY = This->decode_info.yres / 2.54;
-        break;
+    }
+    else
+    {
+        switch (This->decode_info.resolution_unit)
+        {
+        default:
+            FIXME("unknown resolution unit %i\n", This->decode_info.resolution_unit);
+            /* fall through */
+        case 0: /* Not set */
+        case 1: /* Relative measurements */
+        case 2: /* Inch */
+            *pDpiX = This->decode_info.xres;
+            *pDpiY = This->decode_info.yres;
+            break;
+        case 3: /* Centimeter */
+            *pDpiX = This->decode_info.xres * 2.54;
+            *pDpiY = This->decode_info.yres * 2.54;
+            break;
+        }
     }
 
     TRACE("(%p) <-- %f,%f unit=%i\n", iface, *pDpiX, *pDpiY, This->decode_info.resolution_unit);
@@ -1441,6 +1438,8 @@ typedef struct TiffFrameEncode {
     UINT width, height;
     double xres, yres;
     UINT lines_written;
+    WICColor palette[256];
+    UINT colors;
 } TiffFrameEncode;
 
 static inline TiffFrameEncode *impl_from_IWICBitmapFrameEncode(IWICBitmapFrameEncode *iface)
@@ -1602,10 +1601,24 @@ static HRESULT WINAPI TiffFrameEncode_SetColorContexts(IWICBitmapFrameEncode *if
 }
 
 static HRESULT WINAPI TiffFrameEncode_SetPalette(IWICBitmapFrameEncode *iface,
-    IWICPalette *pIPalette)
+    IWICPalette *palette)
 {
-    FIXME("(%p,%p): stub\n", iface, pIPalette);
-    return WINCODEC_ERR_UNSUPPORTEDOPERATION;
+    TiffFrameEncode *This = impl_from_IWICBitmapFrameEncode(iface);
+    HRESULT hr;
+
+    TRACE("(%p,%p)\n", iface, palette);
+
+    if (!palette) return E_INVALIDARG;
+
+    EnterCriticalSection(&This->parent->lock);
+
+    if (This->initialized)
+        hr = IWICPalette_GetColors(palette, 256, This->palette, &This->colors);
+    else
+        hr = WINCODEC_ERR_NOTINITIALIZED;
+
+    LeaveCriticalSection(&This->parent->lock);
+    return hr;
 }
 
 static HRESULT WINAPI TiffFrameEncode_SetThumbnail(IWICBitmapFrameEncode *iface,
@@ -1887,10 +1900,20 @@ static HRESULT WINAPI TiffEncoder_SetColorContexts(IWICBitmapEncoder *iface,
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI TiffEncoder_SetPalette(IWICBitmapEncoder *iface, IWICPalette *pIPalette)
+static HRESULT WINAPI TiffEncoder_SetPalette(IWICBitmapEncoder *iface, IWICPalette *palette)
 {
-    TRACE("(%p,%p)\n", iface, pIPalette);
-    return WINCODEC_ERR_UNSUPPORTEDOPERATION;
+    TiffEncoder *This = impl_from_IWICBitmapEncoder(iface);
+    HRESULT hr;
+
+    TRACE("(%p,%p)\n", iface, palette);
+
+    EnterCriticalSection(&This->lock);
+
+    hr = This->stream ? WINCODEC_ERR_UNSUPPORTEDOPERATION : WINCODEC_ERR_NOTINITIALIZED;
+
+    LeaveCriticalSection(&This->lock);
+
+    return hr;
 }
 
 static HRESULT WINAPI TiffEncoder_SetThumbnail(IWICBitmapEncoder *iface, IWICBitmapSource *pIThumbnail)
@@ -1910,7 +1933,11 @@ static HRESULT WINAPI TiffEncoder_CreateNewFrame(IWICBitmapEncoder *iface,
 {
     TiffEncoder *This = impl_from_IWICBitmapEncoder(iface);
     TiffFrameEncode *result;
-
+    static const PROPBAG2 opts[2] =
+    {
+        { PROPBAG2_TYPE_DATA, VT_UI1, 0, 0, (LPOLESTR)wszTiffCompressionMethod },
+        { PROPBAG2_TYPE_DATA, VT_R4,  0, 0, (LPOLESTR)wszCompressionQuality },
+    };
     HRESULT hr=S_OK;
 
     TRACE("(%p,%p,%p)\n", iface, ppIFrameEncode, ppIEncoderOptions);
@@ -1927,26 +1954,16 @@ static HRESULT WINAPI TiffEncoder_CreateNewFrame(IWICBitmapEncoder *iface,
         hr = E_FAIL;
     }
 
-    if (SUCCEEDED(hr))
+    if (ppIEncoderOptions && SUCCEEDED(hr))
     {
-        PROPBAG2 opts[2]= {{0}};
-        opts[0].pstrName = (LPOLESTR)wszTiffCompressionMethod;
-        opts[0].vt = VT_UI1;
-        opts[0].dwType = PROPBAG2_TYPE_DATA;
-
-        opts[1].pstrName = (LPOLESTR)wszCompressionQuality;
-        opts[1].vt = VT_R4;
-        opts[1].dwType = PROPBAG2_TYPE_DATA;
-
-        hr = CreatePropertyBag2(opts, 2, ppIEncoderOptions);
-
+        hr = CreatePropertyBag2(opts, ARRAY_SIZE(opts), ppIEncoderOptions);
         if (SUCCEEDED(hr))
         {
             VARIANT v;
             VariantInit(&v);
             V_VT(&v) = VT_UI1;
             V_UNION(&v, bVal) = WICTiffCompressionDontCare;
-            hr = IPropertyBag2_Write(*ppIEncoderOptions, 1, opts, &v);
+            hr = IPropertyBag2_Write(*ppIEncoderOptions, 1, (PROPBAG2 *)opts, &v);
             VariantClear(&v);
             if (FAILED(hr))
             {
@@ -1974,6 +1991,7 @@ static HRESULT WINAPI TiffEncoder_CreateNewFrame(IWICBitmapEncoder *iface,
             result->xres = 0.0;
             result->yres = 0.0;
             result->lines_written = 0;
+            result->colors = 0;
 
             IWICBitmapEncoder_AddRef(iface);
             *ppIFrameEncode = &result->IWICBitmapFrameEncode_iface;
