@@ -72,17 +72,18 @@ static CRITICAL_SECTION wined3d_wndproc_cs = {&wined3d_wndproc_cs_debug, -1, 0,
  * where appropriate. */
 struct wined3d_settings wined3d_settings =
 {
-    TRUE,           /* No multithreaded CS by default. */
+    TRUE,           /* Multithreaded CS by default. */
     GLES_AUTO,      /* Try desktop GL first, fallback to ES by default. */
-    MAKEDWORD_VERSION(1, 0), /* Default to legacy OpenGL */
+    FALSE,          /* explicit_gl_version */
+    MAKEDWORD_VERSION(4, 4), /* Default to OpenGL 4.4 */
     TRUE,           /* Use of GLSL enabled by default */
     ORM_FBO,        /* Use FBOs to do offscreen rendering */
     PCI_VENDOR_NONE,/* PCI Vendor ID */
     PCI_DEVICE_NONE,/* PCI Device ID */
     0,              /* The default of memory is set in init_driver_info */
     NULL,           /* No wine logo by default */
+    TRUE,           /* Prefer multisample textures to multisample renderbuffers. */
     ~0u,            /* Don't force a specific sample count by default. */
-    FALSE,          /* No strict draw ordering. */
     FALSE,          /* Don't range check relative addressing indices in float constants. */
     ~0U,            /* No VS shader model limit by default. */
     ~0U,            /* No HS shader model limit by default. */
@@ -107,7 +108,6 @@ struct cxgames_hacks cxgames_hacks =
     FALSE,                      /* Use the absolute value of the POW arguments in ARB */
     FALSE,                      /* INTZ not blacklisted by default */
     WINED3D_MAPBUF_NEVER_NV,    /* Don't use glMapBuffer on dynamic buffers with NV threading */
-    FALSE,                      /* No SM4 with just GLSL 1.30 by default */
 };
 
 struct wined3d * CDECL wined3d_create(DWORD flags)
@@ -115,8 +115,7 @@ struct wined3d * CDECL wined3d_create(DWORD flags)
     struct wined3d *object;
     HRESULT hr;
 
-    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, FIELD_OFFSET(struct wined3d, adapters[1]));
-    if (!object)
+    if (!(object = heap_alloc_zero(FIELD_OFFSET(struct wined3d, adapters[1]))))
     {
         ERR("Failed to allocate wined3d object memory.\n");
         return NULL;
@@ -129,7 +128,7 @@ struct wined3d * CDECL wined3d_create(DWORD flags)
     if (FAILED(hr))
     {
         WARN("Failed to initialize wined3d object, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
+        heap_free(object);
         return NULL;
     }
 
@@ -266,20 +265,18 @@ static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
         }
         if (!get_config_key_dword(hkey, appkey, "MaxVersionGL", &tmpvalue))
         {
-            if (tmpvalue != wined3d_settings.max_gl_version)
-            {
-                ERR_(winediag)("Setting maximum allowed wined3d GL version to %u.%u.\n",
-                        tmpvalue >> 16, tmpvalue & 0xffff);
-                wined3d_settings.max_gl_version = tmpvalue;
-            }
+            ERR_(winediag)("Setting maximum allowed wined3d GL version to %u.%u.\n",
+                    tmpvalue >> 16, tmpvalue & 0xffff);
+            wined3d_settings.explicit_gl_version = TRUE;
+            wined3d_settings.max_gl_version = tmpvalue;
         }
         if ( !get_config_key( hkey, appkey, "UseGLSL", buffer, size) )
         {
             if (!strcmp(buffer,"disabled"))
             {
                 ERR_(winediag)("The GLSL shader backend has been disabled. You get to keep all the pieces if it breaks.\n");
-                TRACE("Use of GL Shading Language disabled\n");
-                wined3d_settings.glslRequested = FALSE;
+                TRACE("Use of GL Shading Language disabled.\n");
+                wined3d_settings.use_glsl = FALSE;
             }
         }
         if ( !get_config_key( hkey, appkey, "hl2_disable_glsl", buffer, size) )
@@ -297,7 +294,7 @@ static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
                 TRACE("Looking for \"%s\"\n", token);
                 if(strstr(cmdline, token)) {
                     TRACE("Disabling GLSL for this HL2 mod\n");
-                    wined3d_settings.glslRequested = FALSE;
+                    wined3d_settings.use_glsl = FALSE;
                     break;
                 }
                 token = strtok(NULL, ";");
@@ -353,20 +350,16 @@ static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
         {
             size_t len = strlen(buffer) + 1;
 
-            wined3d_settings.logo = HeapAlloc(GetProcessHeap(), 0, len);
-            if (!wined3d_settings.logo) ERR("Failed to allocate logo path memory.\n");
-            else memcpy(wined3d_settings.logo, buffer, len);
+            if (!(wined3d_settings.logo = heap_alloc(len)))
+                ERR("Failed to allocate logo path memory.\n");
+            else
+                memcpy(wined3d_settings.logo, buffer, len);
         }
+        if (!get_config_key_dword(hkey, appkey, "MultisampleTextures", &wined3d_settings.multisample_textures))
+            ERR_(winediag)("Setting multisample textures to %#x.\n", wined3d_settings.multisample_textures);
         if (!get_config_key_dword(hkey, appkey, "SampleCount", &wined3d_settings.sample_count))
             ERR_(winediag)("Forcing sample count to %u. This may not be compatible with all applications.\n",
                     wined3d_settings.sample_count);
-        if (!get_config_key(hkey, appkey, "StrictDrawOrdering", buffer, size)
-                && !strcmp(buffer,"enabled"))
-        {
-            ERR_(winediag)("\"StrictDrawOrdering\" is deprecated, please use \"csmt\" instead.");
-            TRACE("Enforcing strict draw ordering.\n");
-            wined3d_settings.strict_draw_ordering = TRUE;
-        }
         if ( !get_config_key( hkey, appkey, "SafeVsConsts", buffer, size) )
         {
             if (!strcmp(buffer,"enable"))
@@ -448,14 +441,6 @@ static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
                 cxgames_hacks.allow_glmapbuffer = WINED3D_MAPBUF_NEVER;
             }
         }
-        if (!get_config_key(hkey, appkey, "GLSL130SM4", buffer, size))
-        {
-            if (!strcmp(buffer, "enabled"))
-            {
-                TRACE("Enabling SM4 on GLSL 1.30.\n");
-                cxgames_hacks.glsl130_sm4 = TRUE;
-            }
-        }
     }
 
     if (appkey) RegCloseKey( appkey );
@@ -485,10 +470,10 @@ static BOOL wined3d_dll_destroy(HINSTANCE hInstDLL)
          * these entries. */
         WARN("Leftover wndproc table entry %p.\n", &wndproc_table.entries[i]);
     }
-    HeapFree(GetProcessHeap(), 0, wndproc_table.entries);
+    heap_free(wndproc_table.entries);
 
-    HeapFree(GetProcessHeap(), 0, wined3d_settings.logo);
-    HeapFree(GetProcessHeap(), 0, cxgames_hacks.disabled_extensions);
+    heap_free(wined3d_settings.logo);
+    heap_free(cxgames_hacks.disabled_extensions);
     UnregisterClassA(WINED3D_OPENGL_WINDOW_CLASS_NAME, hInstDLL);
 
     DeleteCriticalSection(&wined3d_wndproc_cs);
