@@ -41,18 +41,24 @@ struct async
     struct list          queue_entry;     /* entry in async queue list */
     struct list          process_entry;   /* entry in process list */
     struct async_queue  *queue;           /* queue containing this async */
+    struct fd           *fd;              /* fd associated with an unqueued async */
     unsigned int         status;          /* current status */
     struct timeout_user *timeout;
     unsigned int         timeout_status;  /* status to report upon timeout */
     int                  signaled;
-    int                  skip_completion; /* skip completion */
     struct event        *event;
     async_data_t         data;            /* data for async I/O call */
     struct iosb         *iosb;            /* I/O status block */
+    obj_handle_t         wait_handle;     /* pre-allocated wait handle */
+    int                  direct_result;   /* a flag if we're passing result directly from request instead of APC  */
+    struct completion   *completion;      /* completion associated with fd */
+    apc_param_t          comp_key;        /* completion key associated with fd */
+    unsigned int         comp_flags;      /* completion flags */
 };
 
 static void async_dump( struct object *obj, int verbose );
 static int async_signaled( struct object *obj, struct wait_queue_entry *entry );
+static void async_satisfied( struct object * obj, struct wait_queue_entry *entry );
 static void async_destroy( struct object *obj );
 
 static const struct object_ops async_ops =
@@ -63,7 +69,7 @@ static const struct object_ops async_ops =
     add_queue,                 /* add_queue */
     remove_queue,              /* remove_queue */
     async_signaled,            /* signaled */
-    no_satisfied,              /* satisfied */
+    async_satisfied,           /* satisfied */
     no_signal,                 /* signal */
     no_get_fd,                 /* get_fd */
     no_map_access,             /* map_access */
@@ -77,45 +83,9 @@ static const struct object_ops async_ops =
     async_destroy              /* destroy */
 };
 
-
-struct async_queue
-{
-    struct object        obj;             /* object header */
-    struct fd           *fd;              /* file descriptor owning this queue */
-    struct completion   *completion;      /* completion associated with a recently closed file descriptor */
-    apc_param_t          comp_key;        /* completion key associated with a recently closed file descriptor */
-    struct list          queue;           /* queue of async objects */
-};
-
-static void async_queue_dump( struct object *obj, int verbose );
-static void async_queue_destroy( struct object *obj );
-
-static const struct object_ops async_queue_ops =
-{
-    sizeof(struct async_queue),      /* size */
-    async_queue_dump,                /* dump */
-    no_get_type,                     /* get_type */
-    no_add_queue,                    /* add_queue */
-    NULL,                            /* remove_queue */
-    NULL,                            /* signaled */
-    NULL,                            /* satisfied */
-    no_signal,                       /* signal */
-    no_get_fd,                       /* get_fd */
-    no_map_access,                   /* map_access */
-    default_get_sd,                  /* get_sd */
-    default_set_sd,                  /* set_sd */
-    no_lookup_name,                  /* lookup_name */
-    no_link_name,                    /* link_name */
-    NULL,                            /* unlink_name */
-    no_open_file,                    /* open_file */
-    no_close_handle,                 /* close_handle */
-    async_queue_destroy              /* destroy */
-};
-
-
 static inline void async_reselect( struct async *async )
 {
-    if (async->queue && async->queue->fd) fd_reselect_async( async->queue->fd, async->queue );
+    if (async->queue && async->fd) fd_reselect_async( async->fd, async->queue );
 }
 
 static void async_dump( struct object *obj, int verbose )
@@ -132,6 +102,25 @@ static int async_signaled( struct object *obj, struct wait_queue_entry *entry )
     return async->signaled;
 }
 
+static void async_satisfied( struct object *obj, struct wait_queue_entry *entry )
+{
+    struct async *async = (struct async *)obj;
+    assert( obj->ops == &async_ops );
+
+    if (async->direct_result)
+    {
+        async_set_result( &async->obj, async->iosb->status, async->iosb->result );
+        async->direct_result = 0;
+    }
+
+    /* close wait handle here to avoid extra server round trip */
+    if (async->wait_handle)
+    {
+        close_handle( async->thread->process, async->wait_handle );
+        async->wait_handle = 0;
+    }
+}
+
 static void async_destroy( struct object *obj )
 {
     struct async *async = (struct async *)obj;
@@ -143,37 +132,16 @@ static void async_destroy( struct object *obj )
     {
         list_remove( &async->queue_entry );
         async_reselect( async );
-        release_object( async->queue );
     }
+    else if (async->fd) release_object( async->fd );
 
     if (async->timeout) remove_timeout_user( async->timeout );
+    if (async->completion) release_object( async->completion );
     if (async->event) release_object( async->event );
     if (async->iosb) release_object( async->iosb );
     release_object( async->thread );
 }
 
-static void async_queue_dump( struct object *obj, int verbose )
-{
-    struct async_queue *async_queue = (struct async_queue *)obj;
-    assert( obj->ops == &async_queue_ops );
-    fprintf( stderr, "Async queue fd=%p\n", async_queue->fd );
-}
-
-static void async_queue_destroy( struct object *obj )
-{
-    struct async_queue *async_queue = (struct async_queue *)obj;
-    assert( obj->ops == &async_queue_ops );
-    if (async_queue->completion) release_object( async_queue->completion );
-}
-
-void async_skip_completion( struct async *async, int comp_flags )
-{
-    if (!async->data.user) return;
-    if (!(comp_flags & FILE_SKIP_COMPLETION_PORT_ON_SUCCESS)) return;
-    async->skip_completion = (async->status == STATUS_SUCCESS ||
-                              async->status == STATUS_ALERTED);
-}
-
 /* notifies client thread of new status of its async request */
 void async_terminate( struct async *async, unsigned int status )
 {
@@ -189,18 +157,21 @@ void async_terminate( struct async *async, unsigned int status )
     async->status = status;
     if (async->iosb && async->iosb->status == STATUS_PENDING) async->iosb->status = status;
 
-    if (async->data.user)
+    if (!async->direct_result)
     {
-        apc_call_t data;
+        if (async->data.user)
+        {
+            apc_call_t data;
 
-        memset( &data, 0, sizeof(data) );
-        data.type            = APC_ASYNC_IO;
-        data.async_io.user   = async->data.user;
-        data.async_io.sb     = async->data.iosb;
-        data.async_io.status = status;
-        thread_queue_apc( async->thread, &async->obj, &data );
+            memset( &data, 0, sizeof(data) );
+            data.type            = APC_ASYNC_IO;
+            data.async_io.user   = async->data.user;
+            data.async_io.sb     = async->data.iosb;
+            data.async_io.status = status;
+            thread_queue_apc( async->thread->process, async->thread, &async->obj, &data );
+        }
+        else async_set_result( &async->obj, STATUS_SUCCESS, 0 );
     }
-    else async_set_result( &async->obj, STATUS_SUCCESS, 0 );
 
     async_reselect( async );
     if (async->queue) release_object( async );  /* so that it gets destroyed when the async is done */
@@ -215,42 +186,36 @@ static void async_timeout( void *private )
     async_terminate( async, async->timeout_status );
 }
 
-/* create a new async queue for a given fd */
-struct async_queue *create_async_queue( struct fd *fd )
-{
-    struct async_queue *queue = alloc_object( &async_queue_ops );
-
-    if (queue)
-    {
-        queue->fd = fd;
-        queue->completion = NULL;
-        list_init( &queue->queue );
-    }
-    return queue;
-}
-
 /* free an async queue, cancelling all async operations */
 void free_async_queue( struct async_queue *queue )
 {
-    if (!queue) return;
-    if (queue->fd) queue->completion = fd_get_completion( queue->fd, &queue->comp_key );
-    queue->fd = NULL;
-    async_wake_up( queue, STATUS_HANDLES_CLOSED );
-    release_object( queue );
+    struct async *async, *next;
+
+    LIST_FOR_EACH_ENTRY_SAFE( async, next, &queue->queue, struct async, queue_entry )
+    {
+        grab_object( &async->obj );
+        if (!async->completion) async->completion = fd_get_completion( async->fd, &async->comp_key );
+        async->fd = NULL;
+        async_terminate( async, STATUS_HANDLES_CLOSED );
+        async->queue = NULL;
+        release_object( &async->obj );
+    }
 }
 
 void queue_async( struct async_queue *queue, struct async *async )
 {
-    async->queue = (struct async_queue *)grab_object( queue );
+    /* fd will be set to NULL in free_async_queue when fd is destroyed */
+    release_object( async->fd );
 
+    async->queue = queue;
     grab_object( async );
     list_add_tail( &queue->queue, &async->queue_entry );
 
-    if (queue->fd) set_fd_signaled( queue->fd, 0 );
+    set_fd_signaled( async->fd, 0 );
 }
 
 /* create an async on a given queue of a fd */
-struct async *create_async( struct thread *thread, const async_data_t *data, struct iosb *iosb )
+struct async *create_async( struct fd *fd, struct thread *thread, const async_data_t *data, struct iosb *iosb )
 {
     struct event *event = NULL;
     struct async *async;
@@ -264,14 +229,18 @@ struct async *create_async( struct thread *thread, const async_data_t *data, str
         return NULL;
     }
 
-    async->thread  = (struct thread *)grab_object( thread );
-    async->event   = event;
-    async->status  = STATUS_PENDING;
-    async->data    = *data;
-    async->timeout = NULL;
-    async->queue   = NULL;
-    async->signaled = 0;
-    async->skip_completion = 0;
+    async->thread        = (struct thread *)grab_object( thread );
+    async->event         = event;
+    async->status        = STATUS_PENDING;
+    async->data          = *data;
+    async->timeout       = NULL;
+    async->queue         = NULL;
+    async->fd            = (struct fd *)grab_object( fd );
+    async->signaled      = 0;
+    async->wait_handle   = 0;
+    async->direct_result = 0;
+    async->completion    = fd_get_completion( fd, &async->comp_key );
+    async->comp_flags    = 0;
 
     if (iosb) async->iosb = (struct iosb *)grab_object( iosb );
     else async->iosb = NULL;
@@ -279,9 +248,84 @@ struct async *create_async( struct thread *thread, const async_data_t *data, str
     list_add_head( &thread->process->asyncs, &async->process_entry );
     if (event) reset_event( event );
 
+    if (async->completion && data->apc)
+    {
+        release_object( async );
+        set_error( STATUS_INVALID_PARAMETER );
+        return NULL;
+    }
+
     return async;
 }
 
+/* create an async associated with iosb for async-based requests
+ * returned async must be passed to async_handoff */
+struct async *create_request_async( struct fd *fd, unsigned int comp_flags, const async_data_t *data )
+{
+    struct async *async;
+    struct iosb *iosb;
+
+    if (!(iosb = create_iosb( get_req_data(), get_req_data_size(), get_reply_max_size() )))
+        return NULL;
+
+    async = create_async( fd, current, data, iosb );
+    release_object( iosb );
+    if (async)
+    {
+        if (!(async->wait_handle = alloc_handle( current->process, async, SYNCHRONIZE, 0 )))
+        {
+            release_object( async );
+            return NULL;
+        }
+        async->direct_result = 1;
+        async->comp_flags = comp_flags;
+    }
+    return async;
+}
+
+/* return async object status and wait handle to client */
+obj_handle_t async_handoff( struct async *async, int success, data_size_t *result )
+{
+    if (!success)
+    {
+        close_handle( async->thread->process, async->wait_handle );
+        async->wait_handle = 0;
+        return 0;
+    }
+
+    if (get_error() != STATUS_PENDING)
+    {
+        /* status and data are already set and returned */
+        async_terminate( async, get_error() );
+    }
+    else if (async->iosb->status != STATUS_PENDING)
+    {
+        /* result is already available in iosb, return it */
+        if (async->iosb->out_data)
+        {
+            set_reply_data_ptr( async->iosb->out_data, async->iosb->out_size );
+            async->iosb->out_data = NULL;
+        }
+    }
+
+    if (async->iosb->status != STATUS_PENDING)
+    {
+        if (result) *result = async->iosb->result;
+        async->signaled = 1;
+    }
+    else
+    {
+        async->direct_result = 0;
+        if (!async_is_blocking( async ))
+        {
+            close_handle( async->thread->process, async->wait_handle);
+            async->wait_handle = 0;
+        }
+    }
+    set_error( async->iosb->status );
+    return async->wait_handle;
+}
+
 /* set the timeout of an async operation */
 void async_set_timeout( struct async *async, timeout_t timeout, unsigned int status )
 {
@@ -291,22 +335,11 @@ void async_set_timeout( struct async *async, timeout_t timeout, unsigned int sta
     async->timeout_status = status;
 }
 
-static void add_async_completion( struct async_queue *queue, apc_param_t cvalue, unsigned int status,
+static void add_async_completion( struct async *async, apc_param_t cvalue, unsigned int status,
                                   apc_param_t information )
 {
-    if (queue->fd)
-    {
-        apc_param_t ckey;
-        struct completion *completion = fd_get_completion( queue->fd, &ckey );
-
-        if (completion)
-        {
-            add_completion( completion, ckey, cvalue, status, information );
-            release_object( completion );
-        }
-    }
-    else if (queue->completion) add_completion( queue->completion, queue->comp_key,
-                                                cvalue, status, information );
+    if (async->fd && !async->completion) async->completion = fd_get_completion( async->fd, &async->comp_key );
+    if (async->completion) add_completion( async->completion, async->comp_key, cvalue, status, information );
 }
 
 /* store the result of the client-side async callback */
@@ -336,8 +369,6 @@ void async_set_result( struct object *obj, unsigned int status, apc_param_t tota
         async->status = status;
         if (status == STATUS_MORE_PROCESSING_REQUIRED) return;  /* don't report the completion */
 
-        if (async->queue && !async->data.apc && async->data.apc_context && !async->skip_completion)
-            add_async_completion( async->queue, async->data.apc_context, status, total );
         if (async->data.apc)
         {
             apc_call_t data;
@@ -347,19 +378,22 @@ void async_set_result( struct object *obj, unsigned int status, apc_param_t tota
             data.user.args[0] = async->data.apc_context;
             data.user.args[1] = async->data.iosb;
             data.user.args[2] = 0;
-            thread_queue_apc( async->thread, NULL, &data );
+            thread_queue_apc( NULL, async->thread, NULL, &data );
+        }
+        else if (async->data.apc_context && (!async->direct_result ||
+                 !(async->comp_flags & COMPLETION_SKIP_ON_SUCCESS)))
+        {
+            add_async_completion( async, async->data.apc_context, status, total );
         }
-        if (async->event) set_event( async->event );
-        else if (async->queue && async->queue->fd) set_fd_signaled( async->queue->fd, 1 );
-        async->signaled = 1;
-        wake_up( &async->obj, 0 );
-    }
-}
 
-/* check if there are any queued async operations */
-int async_queued( struct async_queue *queue )
-{
-    return queue && list_head( &queue->queue );
+        if (async->event) set_event( async->event );
+        else if (async->fd) set_fd_signaled( async->fd, 1 );
+        if (!async->signaled)
+        {
+            async->signaled = 1;
+            wake_up( &async->obj, 0 );
+        }
+    }
 }
 
 /* check if an async operation is waiting to be alerted */
@@ -368,7 +402,6 @@ int async_waiting( struct async_queue *queue )
     struct list *ptr;
     struct async *async;
 
-    if (!queue) return 0;
     if (!(ptr = list_head( &queue->queue ))) return 0;
     async = LIST_ENTRY( ptr, struct async, queue_entry );
     return async->status == STATUS_PENDING;
@@ -383,7 +416,7 @@ restart:
     LIST_FOR_EACH_ENTRY( async, &process->asyncs, struct async, process_entry )
     {
         if (async->status == STATUS_CANCELLED) continue;
-        if ((!obj || (async->queue && async->queue->fd && get_fd_user( async->queue->fd ) == obj)) &&
+        if ((!obj || (async->fd && get_fd_user( async->fd ) == obj)) &&
             (!thread || async->thread == thread) &&
             (!iosb || async->data.iosb == iosb))
         {
@@ -405,8 +438,6 @@ void async_wake_up( struct async_queue *queue, unsigned int status )
 {
     struct list *ptr, *next;
 
-    if (!queue) return;
-
     LIST_FOR_EACH_SAFE( ptr, next, &queue->queue )
     {
         struct async *async = LIST_ENTRY( ptr, struct async, queue_entry );
@@ -491,7 +522,7 @@ int async_is_blocking( struct async *async )
 struct async *find_pending_async( struct async_queue *queue )
 {
     struct async *async;
-    if (queue) LIST_FOR_EACH_ENTRY( async, &queue->queue, struct async, queue_entry )
+    LIST_FOR_EACH_ENTRY( async, &queue->queue, struct async, queue_entry )
         if (async->status == STATUS_PENDING) return (struct async *)grab_object( async );
     return NULL;
 }
